import os
import inspect
from streamlink.utils import load_module

__all__ = ["load_support_plugin"]


def load_support_plugin(name):
    """Loads a plugin from the same directory as the calling plugin.

    The path used is extracted from the last call in module scope,
    therefore this must be called only from module level in the
    originating plugin or the correct plugin path will not be found.

    """

    # Get the path of the caller module
    stack = list(filter(lambda f: f[3] == "<module>", inspect.stack()))
    prev_frame = stack[0]
    path = os.path.dirname(prev_frame[1])

    # Major hack. If we are frozen by bbfreeze the stack trace will
    # contain relative paths. We therefore use the __file__ variable
    # in this module to correct it.
    if not os.path.isabs(path):
        prefix = os.path.normpath(__file__ + "../../../../../")
        path = os.path.join(prefix, path)

    return load_module(name, path)



from streamlink.plugins.stadium import Stadium
import unittest


class TestPluginStadium(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(Stadium.can_handle_url("http://www.watchstadium.com/live"))
        self.assertTrue(Stadium.can_handle_url("https://www.watchstadium.com/live"))
        self.assertTrue(Stadium.can_handle_url("https://watchstadium.com/live"))
        self.assertTrue(Stadium.can_handle_url("http://watchstadium.com/live"))

        # shouldn't match
        self.assertFalse(Stadium.can_handle_url("http://www.watchstadium.com/anything/else"))
        self.assertFalse(Stadium.can_handle_url("http://www.tvcatchup.com/"))
        self.assertFalse(Stadium.can_handle_url("http://www.youtube.com/"))



import os
import sys
import inspect

is_py2 = (sys.version_info[0] == 2)
is_py3 = (sys.version_info[0] == 3)
is_py33 = (sys.version_info[0] == 3 and sys.version_info[1] == 3)
is_win32 = os.name == "nt"

# win/nix compatible devnull
try:
    from subprocess import DEVNULL

    def devnull():
        return DEVNULL
except ImportError:
    def devnull():
        return open(os.path.devnull, 'w')

if is_py2:
    _str = str
    str = unicode
    range = xrange
    from itertools import izip

    def bytes(b, enc="ascii"):
        return _str(b)

elif is_py3:
    bytes = bytes
    str = str
    range = range
    izip = zip

try:
    from urllib.parse import (
        urlparse, urlunparse, urljoin, quote, unquote, unquote_plus, parse_qsl, urlencode, urlsplit, urlunsplit
    )
    import queue
except ImportError:
    from urlparse import urlparse, urlunparse, urljoin, parse_qsl, urlsplit, urlunsplit
    from urllib import quote, unquote, unquote_plus, urlencode
    import Queue as queue

try:
    from shutil import which
except ImportError:
    from backports.shutil_which import which

try:
    from html import unescape as html_unescape
except ImportError:
    from HTMLParser import HTMLParser
    html_unescape = unescape = HTMLParser().unescape


getargspec = getattr(inspect, "getfullargspec", inspect.getargspec)


__all__ = ["is_py2", "is_py3", "is_py33", "is_win32", "str", "bytes",
           "urlparse", "urlunparse", "urljoin", "parse_qsl", "quote",
           "unquote", "unquote_plus", "queue", "range", "urlencode", "devnull", "which",
           "izip", "urlsplit", "urlunsplit", "getargspec", "html_unescape"]



import re

from streamlink.plugin import Plugin
from streamlink.plugins.brightcove import BrightcovePlayer
from streamlink.stream import RTMPStream


class AlJazeeraEnglish(Plugin):
    url_re = re.compile(r"https?://(?:\w+\.)?aljazeera\.com")
    account_id = 665003303001
    render_re = re.compile(r'''RenderPagesVideo\((?P<q>['"])(?P<id>\d+)(?P=q)''')  # VOD
    video_id_re = re.compile(r'''videoId=(?P<id>\d+)["']''')  # Live

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_streams(self):
        res = self.session.http.get(self.url)

        # check two different styles to include the video id in the page
        video_id_m = self.render_re.search(res.text) or self.video_id_re.search(res.text)
        video_id = video_id_m and video_id_m.group("id")

        if not video_id:
            self.logger.error("Could not find a video ID on this page")
            return

        # Use BrightcovePlayer class to get the streams
        self.logger.debug("Found video ID: {0}", video_id)
        bp = BrightcovePlayer(self.session, self.account_id)

        for q, s in bp.get_streams(video_id):
            # RTMP Streams are listed, but they do not appear to work
            if not isinstance(s, RTMPStream):
                yield q, s


__plugin__ = AlJazeeraEnglish



import logging
import re

from streamlink.compat import urlparse, parse_qsl
from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents
from streamlink.plugin.api.utils import itertags
from streamlink.stream import HLSStream

log = logging.getLogger(__name__)


class Mjunoon(Plugin):
    url_re = re.compile(r"https?://(?:www\.)?mjunoon\.tv/")

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_streams(self):
        self.session.http.headers.update({"User-Agent": useragents.FIREFOX})
        res = self.session.http.get(self.url)
        for script in itertags(res.text, 'script'):
            if script.attributes.get("id") == "playerScript":
                log.debug("Found the playerScript script tag")
                urlparts = urlparse(script.attributes.get("src"))
                i = 0

                for key, url in parse_qsl(urlparts.query):
                    if key == "streamUrl":
                        i += 1
                        for s in HLSStream.parse_variant_playlist(self.session, url, params=dict(id=i), verify=False).items():
                            yield s


__plugin__ = Mjunoon



import sys
import tempfile
import os.path

import datetime

import streamlink.cache

from shutil import rmtree
import unittest
from tests.mock import patch

is_py2 = (sys.version_info[0] == 2)


class TestCache(unittest.TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp("streamlink-test")

        streamlink.cache.cache_dir = self.tmp_dir
        self.cache = streamlink.cache.Cache("cache.json")

    def tearDown(self):
        rmtree(self.tmp_dir)

    def test_get_no_file(self):
        self.assertEqual(self.cache.get("missing-value"), None)
        self.assertEqual(self.cache.get("missing-value", default="default"), "default")

    def test_put_get(self):
        self.cache.set("value", 1)
        self.assertEqual(self.cache.get("value"), 1)

    def test_put_get_prefix(self):
        self.cache.key_prefix = "test"
        self.cache.set("value", 1)
        self.assertEqual(self.cache.get("value"), 1)

    def test_key_prefix(self):
        self.cache.key_prefix = "test"
        self.cache.set("value", 1)
        self.assertTrue("test:value" in self.cache._cache)
        self.assertEqual(1, self.cache._cache["test:value"]["value"])

    @patch('os.path.exists', return_value=True)
    def test_load_fail(self, exists_mock):
        if is_py2:
            patch('__builtin__.open', side_effect=IOError)
        else:
            patch('streamlink.cache.open', side_effect=IOError)
        self.cache._load()
        self.assertEqual({}, self.cache._cache)

    def test_expired(self):
        self.cache.set("value", 10, expires=-20)
        self.assertEqual(None, self.cache.get("value"))

    def test_expired_at_before(self):
        self.cache.set("value", 10, expires_at=datetime.datetime.now() - datetime.timedelta(seconds=20))
        self.assertEqual(None, self.cache.get("value"))

    def test_expired_at_after(self):
        self.cache.set("value", 10, expires_at=datetime.datetime.now() + datetime.timedelta(seconds=20))
        self.assertEqual(10, self.cache.get("value"))

    def test_not_expired(self):
        self.cache.set("value", 10, expires=20)
        self.assertEqual(10, self.cache.get("value"))

    def test_create_directory(self):
        try:
            streamlink.cache.cache_dir = os.path.join(tempfile.gettempdir(), "streamlink-test")
            cache = streamlink.cache.Cache("cache.json")
            self.assertFalse(os.path.exists(cache.filename))
            cache.set("value", 10)
            self.assertTrue(os.path.exists(cache.filename))
        finally:
            rmtree(streamlink.cache.cache_dir, ignore_errors=True)

    @patch('os.makedirs', side_effect=OSError)
    def test_create_directory_fail(self, makedirs):
        try:
            streamlink.cache.cache_dir = os.path.join(tempfile.gettempdir(), "streamlink-test")
            cache = streamlink.cache.Cache("cache.json")
            self.assertFalse(os.path.exists(cache.filename))
            cache.set("value", 10)
            self.assertFalse(os.path.exists(cache.filename))
        finally:
            rmtree(streamlink.cache.cache_dir, ignore_errors=True)

    def test_get_all(self):
        self.cache.set("test1", 1)
        self.cache.set("test2", 2)

        self.assertDictEqual(
            {"test1": 1, "test2": 2},
            self.cache.get_all())

    def test_get_all_prefix(self):
        self.cache.set("test1", 1)
        self.cache.set("test2", 2)
        self.cache.key_prefix = "test"
        self.cache.set("test3", 3)
        self.cache.set("test4", 4)

        self.assertDictEqual(
            {"test3": 3, "test4": 4},
            self.cache.get_all())

    def test_get_all_prune(self):
        self.cache.set("test1", 1)
        self.cache.set("test2", 2, -1)

        self.assertDictEqual(
            {"test1": 1},
            self.cache.get_all())



import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HTTPStream
from streamlink.utils import parse_json


class RadioNet(Plugin):
    _url_re = re.compile(r"https?://(\w+)\.radio\.(net|at|de|dk|es|fr|it|pl|pt|se)")
    _stream_data_re = re.compile(r'\bstation\s*:\s*(\{.+\}),?\s*')

    _stream_schema = validate.Schema(
        validate.transform(_stream_data_re.search),
        validate.any(
            None,
            validate.all(
                validate.get(1),
                validate.transform(parse_json),
                {
                    'type': validate.text,
                    'streams': validate.all([{
                        'url': validate.url()
                    }])
                },
            )
        )
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url)

    def _get_streams(self):
        streams = self.session.http.get(self.url, schema=self._stream_schema)
        if streams is None:
            return

        # Ignore non-radio streams (podcasts...)
        if streams['type'] != 'STATION':
            return

        stream_urls = set()
        for stream in streams['streams']:
            stream_urls.add(stream['url'])

        for url in stream_urls:
            yield 'live', HTTPStream(self.session, url)


__plugin__ = RadioNet



import unittest  # noqa: F401

import pytest

from streamlink.plugins.viutv import ViuTV


class TestPluginViuTV:
    valid_urls = [
        ("https://viu.tv/ch/99",),
    ]
    invalid_urls = [
        ("https://viu.tv/encore/king-maker-ii/king-maker-iie4fuk-hei-baan-hui-ji-on-ji-sun-dang-cheung",),
        ("https://www.youtube.com",)
    ]

    @pytest.mark.parametrize(["url"], valid_urls)
    def test_can_handle_url(self, url):
        assert ViuTV.can_handle_url(url), "url should be handled"

    @pytest.mark.parametrize(["url"], invalid_urls)
    def test_can_handle_url_negative(self, url):
        assert not ViuTV.can_handle_url(url), "url should not be handled"



import logging
import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents, validate
from streamlink.stream import HLSStream, RTMPStream, HTTPStream

log = logging.getLogger(__name__)


class App17(Plugin):
    _url_re = re.compile(r"https://17.live/live/(?P<channel>[^/&?]+)")
    API_URL = "https://api-dsa.17app.co/api/v1/lives/{0}/viewers/alive"

    _api_schema = validate.Schema(
        {
            "rtmpUrls": [{
                validate.optional("provider"): validate.any(int, None),
                "url": validate.url(),
            }],
        },
        validate.get("rtmpUrls"),
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def _get_streams(self):
        match = self._url_re.match(self.url)
        channel = match.group("channel")

        self.session.http.headers.update({'User-Agent': useragents.CHROME, 'Referer': self.url})

        data = '{"liveStreamID":"%s"}' % (channel)

        try:
            res = self.session.http.post(self.API_URL.format(channel), data=data)
            res_json = self.session.http.json(res, schema=self._api_schema)
            log.trace("{0!r}".format(res_json))
            http_url = res_json[0]["url"]
        except Exception as e:
            log.info("Stream currently unavailable.")
            log.debug(str(e))
            return

        https_url = http_url.replace("http:", "https:")
        yield "live", HTTPStream(self.session, https_url)

        if 'pull-rtmp' in http_url:
            rtmp_url = http_url.replace("http:", "rtmp:").replace(".flv", "")
            stream = RTMPStream(self.session, {
                "rtmp": rtmp_url,
                "live": True,
                "pageUrl": self.url,
            })
            yield "live", stream

        if 'wansu-' in http_url:
            hls_url = http_url.replace(".flv", "/playlist.m3u8")
        else:
            hls_url = http_url.replace("live-hdl", "live-hls").replace(".flv", ".m3u8")

        s = HLSStream.parse_variant_playlist(self.session, hls_url)
        if not s:
            yield "live", HLSStream(self.session, hls_url)
        else:
            if len(s) == 1:
                for _n, _s in s.items():
                    yield "live", _s
            else:
                for _s in s.items():
                    yield _s


__plugin__ = App17



import re

from streamlink.plugin import Plugin
from streamlink.plugin.plugin import parse_url_params, LOW_PRIORITY, NORMAL_PRIORITY, NO_PRIORITY
from streamlink.stream import HDSStream
from streamlink.utils import update_scheme
from streamlink.compat import urlparse


class HDSPlugin(Plugin):
    _url_re = re.compile(r"(hds://)?(.+(?:\.f4m)?.*)")

    @classmethod
    def priority(cls, url):
        """
        Returns LOW priority if the URL is not prefixed with hds:// but ends with
        .f4m and return NORMAL priority if the URL is prefixed.
        :param url: the URL to find the plugin priority for
        :return: plugin priority for the given URL
        """
        m = cls._url_re.match(url)
        if m:
            prefix, url = cls._url_re.match(url).groups()
            url_path = urlparse(url).path
            if prefix is None and url_path.endswith(".f4m"):
                return LOW_PRIORITY
            elif prefix is not None:
                return NORMAL_PRIORITY
        return NO_PRIORITY

    @classmethod
    def can_handle_url(cls, url):
        m = cls._url_re.match(url)
        if m:
            url_path = urlparse(m.group(2)).path
            return m.group(1) is not None or url_path.endswith(".f4m")

    def _get_streams(self):
        url, params = parse_url_params(self.url)

        urlnoproto = self._url_re.match(url).group(2)
        urlnoproto = update_scheme("http://", urlnoproto)

        return HDSStream.parse_manifest(self.session, urlnoproto, **params)


__plugin__ = HDSPlugin



import re
import time

from streamlink.cache import Cache
from streamlink.exceptions import PluginError
from streamlink.plugin import Plugin
from streamlink.plugin import PluginArguments, PluginArgument
from streamlink.plugin.api import useragents
from streamlink.stream import HLSStream
from streamlink.utils import update_scheme


class ABweb(Plugin):
    '''BIS Livestreams of french AB Groupe
       http://www.abweb.com/BIS-TV-Online/
    '''

    login_url = 'http://www.abweb.com/BIS-TV-Online/Default.aspx'

    _url_re = re.compile(r'https?://(?:www\.)?abweb\.com/BIS-TV-Online/bistvo-tele-universal.aspx', re.IGNORECASE)
    _hls_re = re.compile(r'''["']file["']:\s?["'](?P<url>[^"']+\.m3u8[^"']+)["']''')
    _iframe_re = re.compile(r'''<iframe[^>]+src=["'](?P<url>[^"']+)["']''')

    _input_re = re.compile(r'''(<input[^>]+>)''')
    _name_re = re.compile(r'''name=["']([^"']*)["']''')
    _value_re = re.compile(r'''value=["']([^"']*)["']''')

    expires_time = 3600 * 24

    arguments = PluginArguments(
        PluginArgument(
            "username",
            required=True,
            requires=["password"],
            metavar="USERNAME",
            help="""
            The username associated with your ABweb account, required to access any
            ABweb stream.
            """,
            prompt="Enter ABweb username"
        ),
        PluginArgument(
            "password",
            sensitive=True,
            metavar="PASSWORD",
            help="A ABweb account password to use with --abweb-username.",
            prompt="Enter ABweb password"
        ),
        PluginArgument(
            "purge-credentials",
            action="store_true",
            help="""
            Purge cached ABweb credentials to initiate a new session and
            reauthenticate.
            """)
    )

    def __init__(self, url):
        super(ABweb, self).__init__(url)
        self._session_attributes = Cache(filename='plugin-cache.json', key_prefix='abweb:attributes')
        self._authed = self._session_attributes.get('ASP.NET_SessionId') and self._session_attributes.get('.abportail1')
        self._expires = self._session_attributes.get('expires', time.time() + self.expires_time)

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def set_expires_time_cache(self):
        expires = time.time() + self.expires_time
        self._session_attributes.set('expires', expires, expires=self.expires_time)

    def get_iframe_url(self):
        self.logger.debug('search for an iframe')
        res = self.session.http.get(self.url)
        m = self._iframe_re.search(res.text)
        if not m:
            raise PluginError('No iframe found.')

        iframe_url = m.group('url')
        iframe_url = update_scheme('http://', iframe_url)
        self.logger.debug('IFRAME URL={0}'.format(iframe_url))
        return iframe_url

    def get_hls_url(self, iframe_url):
        self.logger.debug('search for hls url')
        res = self.session.http.get(iframe_url)
        m = self._hls_re.search(res.text)
        if not m:
            raise PluginError('No playlist found.')

        return m and m.group('url')

    def _login(self, username, password):
        '''login and update cached cookies'''
        self.logger.debug('login ...')

        res = self.session.http.get(self.login_url)
        input_list = self._input_re.findall(res.text)
        if not input_list:
            raise PluginError('Missing input data on login website.')

        data = {}
        for _input_data in input_list:
            try:
                _input_name = self._name_re.search(_input_data).group(1)
            except AttributeError:
                continue

            try:
                _input_value = self._value_re.search(_input_data).group(1)
            except AttributeError:
                _input_value = ''

            data[_input_name] = _input_value

        login_data = {
            'ctl00$Login1$UserName': username,
            'ctl00$Login1$Password': password,
            'ctl00$Login1$LoginButton.x': '0',
            'ctl00$Login1$LoginButton.y': '0'
        }
        data.update(login_data)

        res = self.session.http.post(self.login_url, data=data)

        for cookie in self.session.http.cookies:
            self._session_attributes.set(cookie.name, cookie.value, expires=3600 * 24)

        if self._session_attributes.get('ASP.NET_SessionId') and self._session_attributes.get('.abportail1'):
            self.logger.debug('New session data')
            self.set_expires_time_cache()
            return True
        else:
            self.logger.error('Failed to login, check your username/password')
            return False

    def _get_streams(self):
        self.session.http.headers.update({
            'User-Agent': useragents.CHROME,
            'Referer': 'http://www.abweb.com/BIS-TV-Online/bistvo-tele-universal.aspx'
        })

        login_username = self.get_option('username')
        login_password = self.get_option('password')

        if self.options.get('purge_credentials'):
            self._session_attributes.set('ASP.NET_SessionId', None, expires=0)
            self._session_attributes.set('.abportail1', None, expires=0)
            self._authed = False
            self.logger.info('All credentials were successfully removed.')

        if not self._authed and not (login_username and login_password):
            self.logger.error('A login for ABweb is required, use --abweb-username USERNAME --abweb-password PASSWORD')
            return

        if self._authed:
            if self._expires < time.time():
                self.logger.debug('get new cached cookies')
                # login after 24h
                self.set_expires_time_cache()
                self._authed = False
            else:
                self.logger.info('Attempting to authenticate using cached cookies')
                self.session.http.cookies.set('ASP.NET_SessionId', self._session_attributes.get('ASP.NET_SessionId'))
                self.session.http.cookies.set('.abportail1', self._session_attributes.get('.abportail1'))

        if not self._authed and not self._login(login_username, login_password):
            return

        iframe_url = self.get_iframe_url()
        self.session.http.headers.update({'Referer': iframe_url})

        hls_url = self.get_hls_url(iframe_url)
        hls_url = update_scheme(self.url, hls_url)

        self.logger.debug('URL={0}'.format(hls_url))
        variant = HLSStream.parse_variant_playlist(self.session, hls_url)
        if variant:
            for q, s in variant.items():
                yield q, s
        else:
            yield 'live', HLSStream(self.session, hls_url)


__plugin__ = ABweb



import unittest

from streamlink.plugins.powerapp import PowerApp


class TestPluginPowerApp(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://powerapp.com.tr/tv/powertv4k',
            'http://powerapp.com.tr/tv/powerturktv4k',
            'http://powerapp.com.tr/tv/powerEarthTV',
            'http://www.powerapp.com.tr/tvs/powertv'
        ]
        for url in should_match:
            self.assertTrue(PowerApp.can_handle_url(url), url)

    def test_can_handle_url_negative(self):
        should_not_match = [
            'http://powerapp.com.tr/',
        ]
        for url in should_not_match:
            self.assertFalse(PowerApp.can_handle_url(url))



"""Plugin for Denmark's public service channel DR (Danmarks Radio)."""

import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream, HDSStream

LIVE_CHANNELS_API_URL = "https://www.dr.dk/mu-online/api/1.4/channel/{0}"
VOD_API_URL = "https://www.dr.dk/mu/programcard/expanded/{0}"

STREAMING_TYPES = {
    "HDS": HDSStream.parse_manifest,
    "HLS": HLSStream.parse_variant_playlist
}

_url_re = re.compile(r"""
    http(s)?://(www\.)?dr.dk
    (?:
        /[TtVv]+/
        (?:
            live/(?P<channel>[^/]+)
        )?
        (?:
            se/(.+/)?(?P<program>[^/&?]+)
        )?
    )
""", re.VERBOSE)

_channels_schema = validate.Schema(
    {
        "StreamingServers": validate.all(
            [{
                "LinkType": validate.text,
                "Qualities": [
                    validate.all(
                        {
                            "Streams": validate.all(
                                [
                                    validate.all(
                                        {"Stream": validate.text},
                                        validate.get("Stream")
                                    )
                                ],
                                validate.get(0)
                            )
                        },
                        validate.get("Streams")
                    )
                ],
                "Server": validate.text
            }],
            validate.filter(lambda s: s["LinkType"] in STREAMING_TYPES)
        )
    },
    validate.get("StreamingServers", {})
)

_video_schema = validate.Schema(
    {"Data": [{
        "Assets": validate.all(
            [{validate.optional("Links"): validate.all(
                [{
                    "Target": validate.text,
                    "Uri": validate.text
                }],
                validate.filter(lambda l: l["Target"] in STREAMING_TYPES)
            )}],
            validate.filter(lambda a: "Links" in a)
        )
    }]},
    validate.get("Data", {}),
    validate.get(0, {}),
    validate.get("Assets", {}),
    validate.get(0, {}),
    validate.get("Links", []),
)


class DRDK(Plugin):
    @classmethod
    def can_handle_url(cls, url):
        return _url_re.match(url)

    def _get_streams(self):
        match = _url_re.match(self.url)
        if not match:
            return

        if match.group("channel"):
            return self._get_live_streams(match.group("channel"))
        else:
            return self._get_vod_streams(match.group("program"))

    def _get_vod_streams(self, program):
        res = self.session.http.get(VOD_API_URL.format(program))
        video = self.session.http.json(res, schema=_video_schema)

        streams = {}
        for link in video:
            type = link["Target"]
            url = link["Uri"]
            parser = STREAMING_TYPES[type]

            try:
                streams.update(parser(self.session, url))
            except IOError as err:
                self.logger.error("Failed to load {0} streams: {1}", type, err)

        return streams

    def _get_live_streams(self, slug):
        res = self.session.http.get(LIVE_CHANNELS_API_URL.format(slug))
        servers = self.session.http.json(res, schema=_channels_schema)
        return self._parse_streaming_servers(servers)

    def _parse_streaming_servers(self, servers):
        streams = {}
        for server in servers:
            type = server["LinkType"]
            base_url = server["Server"]
            qualities = server["Qualities"]
            parser = STREAMING_TYPES[type]

            for quality in qualities:
                try:
                    url = "{0}/{1}".format(base_url, quality)
                    streams.update(parser(self.session, url))
                except IOError as err:
                    self.logger.error("Failed to load {0} streams: {1}", type, err)

        return streams


__plugin__ = DRDK



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from streamlink.stream.hls_playlist import load, StreamInfo, Resolution, Media
import unittest
from tests.resources import text


class TestHLSPlaylist(unittest.TestCase):
    def test_load(self):
        with text("hls/test_1.m3u8") as m3u8_fh:
            playlist = load(m3u8_fh.read(), "http://test.se/")

        self.assertEqual(
            playlist.media,
            [
                Media(uri='http://test.se/audio/stereo/en/128kbit.m3u8', type='AUDIO', group_id='stereo',
                      language='en', name='English', default=True, autoselect=True, forced=False,
                      characteristics=None),
                Media(uri='http://test.se/audio/stereo/none/128kbit.m3u8', type='AUDIO', group_id='stereo',
                      language='dubbing', name='Dubbing', default=False, autoselect=True, forced=False,
                      characteristics=None),
                Media(uri='http://test.se/audio/surround/en/320kbit.m3u8', type='AUDIO', group_id='surround',
                      language='en', name='English', default=True, autoselect=True, forced=False,
                      characteristics=None),
                Media(uri='http://test.se/audio/stereo/none/128kbit.m3u8', type='AUDIO', group_id='surround',
                      language='dubbing', name='Dubbing', default=False, autoselect=True, forced=False,
                      characteristics=None),
                Media(uri='http://test.se/subtitles_de.m3u8', type='SUBTITLES', group_id='subs', language='de',
                      name='Deutsch', default=False, autoselect=True, forced=False, characteristics=None),
                Media(uri='http://test.se/subtitles_en.m3u8', type='SUBTITLES', group_id='subs', language='en',
                      name='English', default=True, autoselect=True, forced=False, characteristics=None),
                Media(uri='http://test.se/subtitles_es.m3u8', type='SUBTITLES', group_id='subs', language='es',
                      name='Espanol', default=False, autoselect=True, forced=False, characteristics=None),
                Media(uri='http://test.se/subtitles_fr.m3u8', type='SUBTITLES', group_id='subs', language='fr',
                      name='Français', default=False, autoselect=True, forced=False, characteristics=None)
            ]
        )

        self.assertEqual(
            [p.stream_info for p in playlist.playlists],
            [
                StreamInfo(bandwidth=258157.0, program_id='1', codecs=['avc1.4d400d', 'mp4a.40.2'],
                           resolution=Resolution(width=422, height=180), audio='stereo', video=None,
                           subtitles='subs'),
                StreamInfo(bandwidth=520929.0, program_id='1', codecs=['avc1.4d4015', 'mp4a.40.2'],
                           resolution=Resolution(width=638, height=272), audio='stereo', video=None,
                           subtitles='subs'),
                StreamInfo(bandwidth=831270.0, program_id='1', codecs=['avc1.4d4015', 'mp4a.40.2'],
                           resolution=Resolution(width=638, height=272), audio='stereo', video=None,
                           subtitles='subs'),
                StreamInfo(bandwidth=1144430.0, program_id='1', codecs=['avc1.4d401f', 'mp4a.40.2'],
                           resolution=Resolution(width=958, height=408), audio='surround', video=None,
                           subtitles='subs'),
                StreamInfo(bandwidth=1558322.0, program_id='1', codecs=['avc1.4d401f', 'mp4a.40.2'],
                           resolution=Resolution(width=1277, height=554), audio='surround', video=None,
                           subtitles='subs'),
                StreamInfo(bandwidth=4149264.0, program_id='1', codecs=['avc1.4d4028', 'mp4a.40.2'],
                           resolution=Resolution(width=1921, height=818), audio='surround', video=None,
                           subtitles='subs'),
                StreamInfo(bandwidth=6214307.0, program_id='1', codecs=['avc1.4d4028', 'mp4a.40.2'],
                           resolution=Resolution(width=1921, height=818), audio='surround', video=None,
                           subtitles='subs'),
                StreamInfo(bandwidth=10285391.0, program_id='1', codecs=['avc1.4d4033', 'mp4a.40.2'],
                           resolution=Resolution(width=4096, height=1744), audio='surround', video=None,
                           subtitles='subs')
            ]
        )



# -*- coding: utf-8 -*-
from streamlink.utils import get_filesystem_encoding
from streamlink_cli.output import PlayerOutput
from tests import posix_only, windows_only, py2_only, py3_only
from tests.mock import patch, ANY

UNICODE_TITLE = u"기타치는소율 with UL섬 "


@posix_only
@patch('subprocess.Popen')
def test_output_mpv_unicode_title_posix(popen):
    po = PlayerOutput("mpv", title=UNICODE_TITLE)
    popen().poll.side_effect = lambda: None
    po.open()
    popen.assert_called_with(['mpv', u"--title=" + UNICODE_TITLE, '-'],
                             bufsize=ANY, stderr=ANY, stdout=ANY, stdin=ANY)


@posix_only
@patch('subprocess.Popen')
def test_output_vlc_unicode_title_posix(popen):
    po = PlayerOutput("vlc", title=UNICODE_TITLE)
    popen().poll.side_effect = lambda: None
    po.open()
    popen.assert_called_with(['vlc', u'--input-title-format', UNICODE_TITLE, '-'],
                             bufsize=ANY, stderr=ANY, stdout=ANY, stdin=ANY)


@py2_only
@windows_only
@patch('subprocess.Popen')
def test_output_mpv_unicode_title_windows_py2(popen):
    po = PlayerOutput("mpv.exe", title=UNICODE_TITLE)
    popen().poll.side_effect = lambda: None
    po.open()
    popen.assert_called_with("mpv.exe \"--title=" + UNICODE_TITLE.encode(get_filesystem_encoding()) + "\" -",
                             bufsize=ANY, stderr=ANY, stdout=ANY, stdin=ANY)


@py2_only
@windows_only
@patch('subprocess.Popen')
def test_output_vlc_unicode_title_windows_py2(popen):
    po = PlayerOutput("vlc.exe", title=UNICODE_TITLE)
    popen().poll.side_effect = lambda: None
    po.open()
    popen.assert_called_with("vlc.exe --input-title-format \"" + UNICODE_TITLE.encode(get_filesystem_encoding()) + "\" -",
                             bufsize=ANY, stderr=ANY, stdout=ANY, stdin=ANY)


@py3_only
@windows_only
@patch('subprocess.Popen')
def test_output_mpv_unicode_title_windows_py3(popen):
    po = PlayerOutput("mpv.exe", title=UNICODE_TITLE)
    popen().poll.side_effect = lambda: None
    po.open()
    popen.assert_called_with("mpv.exe \"--title=" + UNICODE_TITLE + "\" -",
                             bufsize=ANY, stderr=ANY, stdout=ANY, stdin=ANY)


@py3_only
@windows_only
@patch('subprocess.Popen')
def test_output_vlc_unicode_title_windows_py3(popen):
    po = PlayerOutput("vlc.exe", title=UNICODE_TITLE)
    popen().poll.side_effect = lambda: None
    po.open()
    popen.assert_called_with("vlc.exe --input-title-format \"" + UNICODE_TITLE + "\" -",
                             bufsize=ANY, stderr=ANY, stdout=ANY, stdin=ANY)



import re

from streamlink.compat import urljoin
from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream

COOKIE_PARAMS = (
    "devicetype=desktop&"
    "preferred-player-odm=hlslink&"
    "preferred-player-live=hlslink"
)

_id_re = re.compile(r"/(?:program|direkte|serie/[^/]+)/([^/]+)")
_url_re = re.compile(r"https?://(tv|radio).nrk.no/")
_api_baseurl_re = re.compile(r'''apiBaseUrl:\s*["'](?P<baseurl>[^"']+)["']''')

_schema = validate.Schema(
    validate.transform(_api_baseurl_re.search),
    validate.any(
        None,
        validate.all(
            validate.get("baseurl"),
            validate.url(
                scheme="http"
            )
        )
    )
)

_mediaelement_schema = validate.Schema({
    "mediaUrl": validate.url(
        scheme="http",
        path=validate.endswith(".m3u8")
    )
})


class NRK(Plugin):
    @classmethod
    def can_handle_url(self, url):
        return _url_re.match(url)

    def _get_streams(self):
        # Get the stream type from the url (tv/radio).
        stream_type = _url_re.match(self.url).group(1).upper()
        cookie = {
            "NRK_PLAYER_SETTINGS_{0}".format(stream_type): COOKIE_PARAMS
        }

        # Construct API URL for this program.
        baseurl = self.session.http.get(self.url, cookies=cookie, schema=_schema)
        program_id = _id_re.search(self.url).group(1)

        # Extract media URL.
        json_url = urljoin(baseurl, "mediaelement/{0}".format(program_id))
        res = self.session.http.get(json_url, cookies=cookie)
        media_element = self.session.http.json(res, schema=_mediaelement_schema)
        media_url = media_element["mediaUrl"]

        return HLSStream.parse_variant_playlist(self.session, media_url)


__plugin__ = NRK



import logging
import re

from streamlink.plugin import Plugin, PluginArgument, PluginArguments
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream
from streamlink.utils import parse_json

log = logging.getLogger(__name__)


class OPENRECtv(Plugin):
    _url_re = re.compile(r"https?://(?:www\.)?openrec.tv/(?:live|movie)/(?P<id>[^/]+)")
    _stores_re = re.compile(r"window.stores\s*=\s*({.*?});", re.DOTALL | re.MULTILINE)
    _config_re = re.compile(r"window.sharedConfig\s*=\s*({.*?});", re.DOTALL | re.MULTILINE)

    api_url = "https://apiv5.openrec.tv/api/v5/movies/{id}/detail"
    login_url = "https://www.openrec.tv/viewapp/v4/mobile/user/login"

    _config_schema = validate.Schema({
        "urls": {
            "apiv5Authorized": validate.url()
        }
    })
    _stores_schema = validate.Schema({
        "moviePageStore": {
            "movieStore": {
                "id": validate.text,
                "title": validate.text,
                "media": {
                    "url": validate.any(None, '', validate.url())
                }
            }
        }
    }, validate.get("moviePageStore"), validate.get("movieStore"))

    _detail_schema = validate.Schema({
        validate.optional("error_message"): validate.text,
        "status": int,
        validate.optional("data"): {
            "type": validate.text,
            "items": [{
                "media": {
                    "url": validate.any(None, validate.url()),
                    "url_dvr": validate.any(None, validate.url())
                }
            }]
        }
    })

    _login_schema = validate.Schema({
        validate.optional("error_message"): validate.text,
        "status": int,
        validate.optional("data"): object
    })

    arguments = PluginArguments(
        PluginArgument(
            "email",
            requires=["password"],
            metavar="EMAIL",
            help="""
            The email associated with your openrectv account,
            required to access any openrectv stream.
            """),
        PluginArgument(
            "password",
            sensitive=True,
            metavar="PASSWORD",
            help="""
            An openrectv account password to use with --openrectv-email.
            """)
    )

    def __init__(self, url):
        super(OPENRECtv, self).__init__(url)
        self._pdata = None
        self._pres = None
        self._pconfig = None

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def login(self, email, password):
        res = self.session.http.post(self.login_url, data={"mail": email, "password": password})
        data = self.session.http.json(res, self._login_schema)
        if data["status"] == 0:
            log.debug("Logged in as {0}".format(data["data"]["user_name"]))
        else:
            log.error("Failed to login: {0}".format(data["error_message"]))
        return data["status"] == 0

    def _get_page(self):
        if not self._pres:
            self._pres = self.session.http.get(self.url)
        return self._pres

    def _get_movie_data(self):
        pres = self._get_page()
        match = self._stores_re.search(pres.text)
        if match:
            self._pdata = parse_json(match.group(1), schema=self._stores_schema)

        return self._pdata

    def _get_page_config(self):
        pres = self._get_page()
        match = self._config_re.search(pres.text)
        if match:
            self._pconfig = parse_json(match.group(1))

        return self._pconfig

    def _get_details(self, id):
        config = self._get_page_config()
        api_url = config["urls"]["apiv5Authorized"]
        url = "{base}/movies/{id}/detail".format(base=api_url, id=id)
        res = self.session.http.get(url, headers={
            "access-token": self.session.http.cookies.get("access_token"),
            "uuid": self.session.http.cookies.get("uuid")
        })
        data = self.session.http.json(res, schema=self._detail_schema)

        if data["status"] == 0:
            log.debug("Got valid detail response")
            return data["data"]
        else:
            log.error("Failed to get video stream: {0}".format(data["error_message"]))

    def get_title(self):
        mdata = self._get_movie_data()
        if mdata:
            return mdata["title"]

    def _get_streams(self):
        mdata = self._get_movie_data()
        if mdata:
            log.debug("Found video: {0} ({1})".format(mdata["title"], mdata["id"]))
            if mdata["media"]["url"]:
                for s in HLSStream.parse_variant_playlist(self.session, mdata["media"]["url"]).items():
                    yield s
            elif self.get_option("email") and self.get_option("password"):
                if self.login(self.get_option("email"), self.get_option("password")):
                    details = self._get_details(mdata["id"])
                    if details:
                        for item in details["items"]:
                            for s in HLSStream.parse_variant_playlist(self.session, item["media"]["url"]).items():
                                yield s
            else:
                log.error("You must login to access this stream")


__plugin__ = OPENRECtv



import re

from streamlink.exceptions import PluginError
from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents
from streamlink.stream import HLSStream
from streamlink.stream import HTTPStream


class TVP(Plugin):
    '''Telewizja Polska S.A.
       http://tvpstream.vod.tvp.pl
    '''

    player_url = 'https://www.tvp.pl/sess/tvplayer.php?object_id={0}&autoplay=true'

    _url_re = re.compile(r'https?://tvpstream\.vod\.tvp\.pl')
    _stream_re = re.compile(r'''src:["'](?P<url>[^"']+\.(?:m3u8|mp4))["']''')
    _video_id_re = re.compile(r'''class=["']tvp_player["'][^>]+data-video-id=["'](?P<video_id>\d+)["']''')

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def get_embed_url(self):
        res = self.session.http.get(self.url)

        m = self._video_id_re.search(res.text)
        if not m:
            raise PluginError('Unable to find a video id')

        video_id = m.group('video_id')
        self.logger.debug('Found video id: {0}'.format(video_id))
        p_url = self.player_url.format(video_id)
        return p_url

    def _get_streams(self):
        self.session.http.headers.update({'User-Agent': useragents.FIREFOX})

        embed_url = self.get_embed_url()
        res = self.session.http.get(embed_url)
        m = self._stream_re.findall(res.text)
        if not m:
            raise PluginError('Unable to find a stream url')

        streams = []
        for url in m:
            self.logger.debug('URL={0}'.format(url))
            if url.endswith('.m3u8'):
                for s in HLSStream.parse_variant_playlist(self.session, url, name_fmt='{pixels}_{bitrate}').items():
                    streams.append(s)
            elif url.endswith('.mp4'):
                streams.append(('vod', HTTPStream(self.session, url)))

        return streams


__plugin__ = TVP



import unittest
from tests.mock import Mock

from streamlink_cli.main import setup_plugin_args
from streamlink.options import Options, Arguments, Argument


class TestOptions(unittest.TestCase):
    def setUp(self):
        self.options = Options({
            "a_default": "default",
            "another-default": "default2"
        })

    def test_options(self):
        self.assertEqual(self.options.get("a_default"), "default")
        self.assertEqual(self.options.get("non_existing"), None)

        self.options.set("a_option", "option")
        self.assertEqual(self.options.get("a_option"), "option")

    def test_options_update(self):
        self.assertEqual(self.options.get("a_default"), "default")
        self.assertEqual(self.options.get("non_existing"), None)

        self.options.update({"a_option": "option"})
        self.assertEqual(self.options.get("a_option"), "option")

    def test_options_name_normalised(self):
        self.assertEqual(self.options.get("a_default"), "default")
        self.assertEqual(self.options.get("a-default"), "default")
        self.assertEqual(self.options.get("another-default"), "default2")
        self.assertEqual(self.options.get("another_default"), "default2")


class TestArgument(unittest.TestCase):
    def test_name(self):
        self.assertEqual(Argument("test-arg").argument_name("plugin"), "--plugin-test-arg")
        self.assertEqual(Argument("test-arg").namespace_dest("plugin"), "plugin_test_arg")
        self.assertEqual(Argument("test-arg").dest, "test_arg")

    def test_name_plugin(self):
        self.assertEqual(Argument("test-arg").argument_name("test_plugin"), "--test-plugin-test-arg")
        self.assertEqual(Argument("test-arg").namespace_dest("test_plugin"), "test_plugin_test_arg")
        self.assertEqual(Argument("test-arg").dest, "test_arg")

    def test_name_override(self):
        self.assertEqual(Argument("test", argument_name="override-name").argument_name("plugin"), "--override-name")
        self.assertEqual(Argument("test", argument_name="override-name").namespace_dest("plugin"), "override_name")
        self.assertEqual(Argument("test", argument_name="override-name").dest, "test")


class TestArguments(unittest.TestCase):
    def test_getter(self):
        test1 = Argument("test1")
        test2 = Argument("test2")
        args = Arguments(test1, test2)

        self.assertEqual(args.get("test1"), test1)
        self.assertEqual(args.get("test2"), test2)
        self.assertEqual(args.get("test3"), None)

    def test_iter(self):
        test1 = Argument("test1")
        test2 = Argument("test2")
        args = Arguments(test1, test2)

        i_args = iter(args)

        self.assertEqual(next(i_args), test1)
        self.assertEqual(next(i_args), test2)

    def test_requires(self):
        test1 = Argument("test1", requires="test2")
        test2 = Argument("test2", requires="test3")
        test3 = Argument("test3")

        args = Arguments(test1, test2, test3)

        self.assertEqual(list(args.requires("test1")), [test2, test3])

    def test_requires_invalid(self):
        test1 = Argument("test1", requires="test2")

        args = Arguments(test1)

        self.assertRaises(KeyError, lambda: list(args.requires("test1")))

    def test_requires_cycle(self):
        test1 = Argument("test1", requires="test2")
        test2 = Argument("test2", requires="test1")

        args = Arguments(test1, test2)

        self.assertRaises(RuntimeError, lambda: list(args.requires("test1")))

    def test_requires_cycle_deep(self):
        test1 = Argument("test1", requires="test-2")
        test2 = Argument("test-2", requires="test3")
        test3 = Argument("test3", requires="test1")

        args = Arguments(test1, test2, test3)

        self.assertRaises(RuntimeError, lambda: list(args.requires("test1")))

    def test_requires_cycle_self(self):
        test1 = Argument("test1", requires="test1")

        args = Arguments(test1)

        self.assertRaises(RuntimeError, lambda: list(args.requires("test1")))


class TestSetupOptions(unittest.TestCase):
    def test_set_defaults(self):
        session = Mock()
        plugin = Mock()
        parser = Mock()

        session.plugins = {"mock": plugin}
        plugin.arguments = Arguments(
            Argument("test1", default="default1"),
            Argument("test2", default="default2"),
            Argument("test3")
        )

        setup_plugin_args(session, parser)

        self.assertEqual(plugin.options.get("test1"), "default1")
        self.assertEqual(plugin.options.get("test2"), "default2")
        self.assertEqual(plugin.options.get("test3"), None)



import unittest

from streamlink.plugins.tga import Tga


class TestPluginTga(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://star.longzhu.com/lpl',
            'http://y.longzhu.com/y123123?from=tonglan2.1',
            'http://star.longzhu.com/123123?from=tonglan4.3',
        ]
        for url in should_match:
            self.assertTrue(Tga.can_handle_url(url))

    def test_can_handle_url_negative(self):
        should_not_match = [
            'https://example.com/index.html',
        ]
        for url in should_not_match:
            self.assertFalse(Tga.can_handle_url(url))



import unittest

from streamlink.plugins.webcast_india_gov import WebcastIndiaGov


class TestPluginWebcastIndiaGov(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(WebcastIndiaGov.can_handle_url("http://webcast.gov.in/ddpunjabi/"))
        self.assertTrue(WebcastIndiaGov.can_handle_url("http://webcast.gov.in/#Channel1"))
        self.assertTrue(WebcastIndiaGov.can_handle_url("http://webcast.gov.in/#Channel3"))

        # shouldn't match
        self.assertFalse(WebcastIndiaGov.can_handle_url("http://meity.gov.in/"))
        self.assertFalse(WebcastIndiaGov.can_handle_url("http://www.nic.in/"))
        self.assertFalse(WebcastIndiaGov.can_handle_url("http://digitalindiaawards.gov.in/"))



import re
from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream


class DingitTV(Plugin):
    """
    Plugin that supports playing streams from DingIt.tv
    """

    # regex to match the site urls
    url_re = re.compile(r"""
        http://www.dingit.tv/(
            highlight/(?P<highlight_id>\d+)|
            channel/(?P<broadcaster>\w+)/(?P<channel_id>\d+)
    )""", re.VERBOSE)

    # flashvars API url and schema
    flashvars_url = "http://www.dingit.tv/api/get_player_flashvars"
    flashvars_schema = validate.Schema({
        u"status": 0,
        u"data": [{
            validate.optional("stream"): validate.text,
            validate.optional("akaurl"): validate.text,
            validate.optional("pereakaurl"): validate.text,
        }]
    },
        validate.get("data"),
        validate.length(1),
        validate.get(0)
    )

    pereakaurl = "http://dingitmedia-vh.akamaihd.net/i/{}/master.m3u8"
    akaurl = "https://dingmedia1-a.akamaihd.net/processed/delivery/{}70f8b7bc-5ed4-336d-609a-2d2cd86288c6.m3u8"

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    @Plugin.broken()
    def _get_streams(self):
        match = self.url_re.match(self.url)

        res = self.session.http.post(
            self.flashvars_url,
            data=dict(
                broadcaster=match.group("broadcaster") or "Verm",
                stream_id=match.group("channel_id") or match.group("highlight_id")
            )
        )

        flashvars = self.session.http.json(res, schema=self.flashvars_schema)

        if flashvars.get("pereakaurl"):
            url = self.pereakaurl.format(flashvars.get("pereakaurl").strip("/"))
            return HLSStream.parse_variant_playlist(self.session, url)

        elif flashvars.get("akaurl"):
            url = self.akaurl.format(flashvars.get("akaurl").strip("/"))
            return HLSStream.parse_variant_playlist(self.session, url)

        elif flashvars.get("stream"):
            self.logger.error("OctoStreams are not currently supported")


__plugin__ = DingitTV



import re

from streamlink.compat import urljoin
from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.plugin.api.utils import parse_json
from streamlink.stream import AkamaiHDStream, HLSStream

_url_re = re.compile(r"http(s)?://(www\.)?livestream.com/")
_stream_config_schema = validate.Schema({
    "event": {
        "stream_info": validate.any({
            "is_live": bool,
            "qualities": [{
                "bitrate": int,
                "height": int
            }],
            validate.optional("play_url"): validate.url(scheme="http"),
            validate.optional("m3u8_url"): validate.url(
                scheme="http",
                path=validate.endswith(".m3u8")
            ),
        }, None)
    },
    validate.optional("playerUri"): validate.text,
    validate.optional("viewerPlusSwfUrl"): validate.url(scheme="http"),
    validate.optional("lsPlayerSwfUrl"): validate.text,
    validate.optional("hdPlayerSwfUrl"): validate.text
})
_smil_schema = validate.Schema(validate.union({
    "http_base": validate.all(
        validate.xml_find("{http://www.w3.org/2001/SMIL20/Language}head/"
                          "{http://www.w3.org/2001/SMIL20/Language}meta"
                          "[@name='httpBase']"),
        validate.xml_element(attrib={
            "content": validate.text
        }),
        validate.get("content")
    ),
    "videos": validate.all(
        validate.xml_findall("{http://www.w3.org/2001/SMIL20/Language}body/"
                             "{http://www.w3.org/2001/SMIL20/Language}switch/"
                             "{http://www.w3.org/2001/SMIL20/Language}video"),
        [
            validate.all(
                validate.xml_element(attrib={
                    "src": validate.text,
                    "system-bitrate": validate.all(
                        validate.text,
                        validate.transform(int)
                    )
                }),
                validate.transform(
                    lambda e: (e.attrib["src"], e.attrib["system-bitrate"])
                )
            )
        ],
    )
}))


class Livestream(Plugin):
    @classmethod
    def default_stream_types(cls, streams):
        return ["akamaihd", "hls"]

    @classmethod
    def can_handle_url(self, url):
        return _url_re.match(url)

    def _get_stream_info(self):
        res = self.session.http.get(self.url)
        match = re.search("window.config = ({.+})", res.text)
        if match:
            config = match.group(1)
            return parse_json(config, "config JSON",
                              schema=_stream_config_schema)

    def _parse_smil(self, url, swf_url):
        res = self.session.http.get(url)
        smil = self.session.http.xml(res, "SMIL config", schema=_smil_schema)

        for src, bitrate in smil["videos"]:
            url = urljoin(smil["http_base"], src)
            yield bitrate, AkamaiHDStream(self.session, url, swf=swf_url)

    def _get_streams(self):
        info = self._get_stream_info()
        if not info:
            return

        stream_info = info["event"]["stream_info"]
        if not (stream_info and stream_info["is_live"]):
            # Stream is not live
            return

        play_url = stream_info.get("play_url")
        if play_url:
            swf_url = (
                info.get("playerUri")
                or info.get("hdPlayerSwfUrl")
                or info.get("lsPlayerSwfUrl")
                or info.get("viewerPlusSwfUrl")
            )
            if swf_url:
                if not swf_url.startswith("http"):
                    if swf_url.startswith("//"):
                        swf_url = "http:" + swf_url
                    else:
                        swf_url = "http://" + swf_url

                # Work around broken SSL.
                swf_url = swf_url.replace("https://", "http://")

            qualities = stream_info["qualities"]
            for bitrate, stream in self._parse_smil(play_url, swf_url):
                name = "{0:d}k".format(int(bitrate / 1000))
                for quality in qualities:
                    if quality["bitrate"] == bitrate:
                        name = "{0}p".format(quality["height"])

                yield name, stream

        m3u8_url = stream_info.get("m3u8_url")
        if m3u8_url:
            streams = HLSStream.parse_variant_playlist(self.session, m3u8_url,
                                                       namekey="pixels")
            # TODO: Replace with "yield from" when dropping Python 2.
            for stream in streams.items():
                yield stream


__plugin__ = Livestream



import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import StreamMapper
from streamlink.plugin.api import validate
from streamlink.plugin.api import useragents
from streamlink.stream import HDSStream
from streamlink.stream import HLSStream
from streamlink.stream import HTTPStream
from streamlink.utils import parse_json, parse_xml


class AdultSwim(Plugin):
    API_URL = "http://www.adultswim.com/videos/api/v2/videos/{id}?fields=stream"
    vod_api = " http://www.adultswim.com/videos/api/v0/assets"

    url_re = re.compile(r"""https?://(?:www\.)?adultswim\.com/videos
            (?:/(streams))?
            (?:/([^/]+))?
            (?:/([^/]+))?
            """, re.VERBOSE)
    _stream_data_re = re.compile(r"(?:__)?AS_INITIAL_DATA(?:__)? = (\{.*?});", re.M | re.DOTALL)

    live_schema = validate.Schema({
        u"streams": {
            validate.text: {u"stream": validate.text,
                            validate.optional(u"isLive"): bool,
                            u"archiveEpisodes": [{
                                u"id": validate.text,
                                u"slug": validate.text,
                            }]}}

    })
    vod_id_schema = validate.Schema({u"show": {u"sluggedVideo": {u"id": validate.text}}},
                                    validate.transform(lambda x: x["show"]["sluggedVideo"]["id"]))
    _api_schema = validate.Schema({
        u'status': u'ok',
        u'data': {u'stream': {
            u'assets': [{u'url': validate.url()}]
        }}
    })
    _vod_api_schema = validate.Schema(
        validate.all(
            validate.xml_findall(".//files/file"),
            [validate.xml_element,
             validate.transform(lambda v: {"bitrate": v.attrib.get("bitrate"), "url": v.text})
             ]
        )
    )

    @classmethod
    def can_handle_url(cls, url):
        match = AdultSwim.url_re.match(url)
        return match is not None

    def _make_hls_hds_stream(self, func, stream, *args, **kwargs):
        return func(self.session, stream["url"], *args, **kwargs)

    def _get_show_streams(self, stream_data, show, episode, platform="desktop"):
        video_id = parse_json(stream_data.group(1), schema=self.vod_id_schema)
        res = self.session.http.get(self.vod_api, params={"platform": platform, "id": video_id})

        # create a unique list of the stream manifest URLs
        streams = []
        urldups = []
        for stream in parse_xml(res.text, schema=self._vod_api_schema):
            if stream["url"] not in urldups:
                streams.append(stream)
                urldups.append(stream["url"])

        mapper = StreamMapper(lambda fmt, strm: strm["url"].endswith(fmt))
        mapper.map(".m3u8", self._make_hls_hds_stream, HLSStream.parse_variant_playlist)
        mapper.map(".f4m", self._make_hls_hds_stream, HDSStream.parse_manifest, is_akamai=True)
        mapper.map(".mp4", lambda s: (s["bitrate"] + "k", HTTPStream(self.session, s["url"])))

        for q, s in mapper(streams):
            yield q, s

    def _get_live_stream(self, stream_data, show, episode=None):
        # parse the stream info as json
        stream_info = parse_json(stream_data.group(1), schema=self.live_schema)
        # get the stream ID
        stream_id = None
        show_info = stream_info[u"streams"][show]

        if episode:
            self.logger.debug("Loading replay of episode: {0}/{1}", show, episode)
            for epi in show_info[u"archiveEpisodes"]:
                if epi[u"slug"] == episode:
                    stream_id = epi[u"id"]
        elif show_info.get("isLive") or not len(show_info[u"archiveEpisodes"]):
            self.logger.debug("Loading LIVE streams for: {0}", show)
            stream_id = show_info[u"stream"]
        else:  # off-air
            if len(show_info[u"archiveEpisodes"]):
                epi = show_info[u"archiveEpisodes"][0]
                self.logger.debug("Loading replay of episode: {0}/{1}", show, epi[u"slug"])
                stream_id = epi[u"id"]
            else:
                self.logger.error("This stream is currently offline")
                return

        if stream_id:
            api_url = self.API_URL.format(id=stream_id)

            res = self.session.http.get(api_url, headers={"User-Agent": useragents.SAFARI_8})
            stream_data = self.session.http.json(res, schema=self._api_schema)

            mapper = StreamMapper(lambda fmt, surl: surl.endswith(fmt))
            mapper.map(".m3u8", HLSStream.parse_variant_playlist, self.session)
            mapper.map(".f4m", HDSStream.parse_manifest, self.session)

            stream_urls = [asset[u"url"] for asset in stream_data[u'data'][u'stream'][u'assets']]
            for q, s in mapper(stream_urls):
                yield q, s

        else:
            self.logger.error("Couldn't find the stream ID for this stream: {0}".format(show))

    def _get_streams(self):
        # get the page
        url_match = self.url_re.match(self.url)
        live_stream, show_name, episode_name = url_match.groups()
        if live_stream:
            show_name = show_name or "live-stream"

        res = self.session.http.get(self.url, headers={"User-Agent": useragents.SAFARI_8})
        # find the big blob of stream info in the page
        stream_data = self._stream_data_re.search(res.text)

        if stream_data:
            if live_stream:
                streams = self._get_live_stream(stream_data, show_name, episode_name)
            else:
                self.logger.debug("Loading VOD streams for: {0}/{1}", show_name, episode_name)
                streams = self._get_show_streams(stream_data, show_name, episode_name)

            # De-dup the streams, some of the mobile streams overlap the desktop streams
            dups = set()
            for q, s in streams:
                if hasattr(s, "args") and "url" in s.args:
                    if s.args["url"] not in dups:
                        yield q, s
                        dups.add(s.args["url"])
                else:
                    yield q, s

        else:
            self.logger.error("Couldn't find the stream data for this stream: {0}".format(show_name))


__plugin__ = AdultSwim



from __future__ import print_function
import re

from streamlink import streams
from streamlink.plugin import Plugin


class StarTV(Plugin):
    url_re = re.compile(r"https?://(?:www\.)?startv.com.tr/canli-yayin")
    iframe_re = re.compile(r'frame .*?src="(https://www.youtube.com/[^"]+)"')

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_streams(self):
        res = self.session.http.get(self.url)
        m = self.iframe_re.search(res.text)

        yt_url = m and m.group(1)
        if yt_url:
            self.logger.debug("Deferring to YouTube plugin with URL: {0}".format(yt_url))
            return streams(yt_url)


__plugin__ = StarTV



import unittest

from streamlink.plugins.bbciplayer import BBCiPlayer


class TestPluginBBCiPlayer(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(
            BBCiPlayer.can_handle_url("http://www.bbc.co.uk/iplayer/episode/b00ymh67/madagascar-1-island-of-marvels"))
        self.assertTrue(BBCiPlayer.can_handle_url("http://www.bbc.co.uk/iplayer/live/bbcone"))

        # shouldn't match
        self.assertFalse(BBCiPlayer.can_handle_url("http://www.tvcatchup.com/"))
        self.assertFalse(BBCiPlayer.can_handle_url("http://www.sportal.bg/sportal_live_tv.php?str=15"))
        self.assertFalse(BBCiPlayer.can_handle_url("http://www.bbc.co.uk/iplayer/"))

    def test_vpid_hash(self):
        self.assertEqual(
            "71c345435589c6ddeea70d6f252e2a52281ecbf3",
            BBCiPlayer._hash_vpid("1234567890")
        )



import re

from streamlink.plugin import Plugin
from streamlink.plugin import PluginArguments, PluginArgument
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream

CHANNEL_API_URL = "http://live.afreecatv.com:8057/afreeca/player_live_api.php"
STREAM_INFO_URLS = "{rmd}/broad_stream_assign.html"

CHANNEL_RESULT_ERROR = 0
CHANNEL_RESULT_OK = 1

QUALITYS = ["original", "hd", "sd"]

QUALITY_WEIGHTS = {
    "original": 1080,
    "hd": 720,
    "sd": 480
}

_url_re = re.compile(r"https?://play\.afreecatv\.com/(?P<username>\w+)(?:/\d+)?")

_channel_schema = validate.Schema(
    {
        "CHANNEL": {
            "RESULT": validate.transform(int),
            validate.optional("BPWD"): validate.text,
            validate.optional("BNO"): validate.text,
            validate.optional("RMD"): validate.text,
            validate.optional("AID"): validate.text,
            validate.optional("CDN"): validate.text
        }
    },
    validate.get("CHANNEL")
)

_stream_schema = validate.Schema(
    {
        validate.optional("view_url"): validate.url(
            scheme=validate.any("rtmp", "http")
        ),
        "stream_status": validate.text
    }
)


class AfreecaTV(Plugin):
    login_url = "https://member.afreecatv.com:8111/login/LoginAction.php"

    arguments = PluginArguments(
        PluginArgument(
            "username",
            requires=["password"],
            metavar="USERNAME",
            help="The username used to register with afreecatv.com."
        ),
        PluginArgument(
            "password",
            sensitive=True,
            metavar="PASSWORD",
            help="A afreecatv.com account password to use with --afreeca-username."
        )
    )

    @classmethod
    def can_handle_url(self, url):
        return _url_re.match(url)

    @classmethod
    def stream_weight(cls, key):
        weight = QUALITY_WEIGHTS.get(key)
        if weight:
            return weight, "afreeca"

        return Plugin.stream_weight(key)

    def _get_channel_info(self, username):
        data = {
            "bid": username,
            "mode": "landing",
            "player_type": "html5"
        }

        res = self.session.http.post(CHANNEL_API_URL, data=data)
        return self.session.http.json(res, schema=_channel_schema)

    def _get_hls_key(self, broadcast, username, quality):
        data = {
            "bid": username,
            "bno": broadcast,
            "pwd": "",
            "quality": quality,
            "type": "pwd"
        }
        res = self.session.http.post(CHANNEL_API_URL, data=data)
        return self.session.http.json(res, schema=_channel_schema)

    def _get_stream_info(self, broadcast, quality, cdn, rmd):
        params = {
            "return_type": cdn,
            "broad_key": "{broadcast}-flash-{quality}-hls".format(**locals())
        }
        res = self.session.http.get(STREAM_INFO_URLS.format(rmd=rmd), params=params)
        return self.session.http.json(res, schema=_stream_schema)

    def _get_hls_stream(self, broadcast, username, quality, cdn, rmd):
        keyjson = self._get_hls_key(broadcast, username, quality)

        if keyjson["RESULT"] != CHANNEL_RESULT_OK:
            return
        key = keyjson["AID"]

        info = self._get_stream_info(broadcast, quality, cdn, rmd)

        if "view_url" in info:
            return HLSStream(self.session, info["view_url"], params=dict(aid=key))

    def _login(self, username, password):
        data = {
            "szWork": "login",
            "szType": "json",
            "szUid": username,
            "szPassword": password,
            "isSaveId": "true",
            "isSavePw": "false",
            "isSaveJoin": "false"
        }

        res = self.session.http.post(self.login_url, data=data)
        res = self.session.http.json(res)
        if res["RESULT"] == 1:
            return True
        else:
            return False

    def _get_streams(self):
        self.session.http.headers.update({"Referer": self.url})
        if not self.session.get_option("hls-segment-ignore-names"):
            ignore_segment = ["preloading"]
            self.session.set_option("hls-segment-ignore-names", ignore_segment)

        login_username = self.get_option("username")
        login_password = self.get_option("password")
        if login_username and login_password:
            self.logger.debug("Attempting login as {0}".format(login_username))
            if self._login(login_username, login_password):
                self.logger.info("Successfully logged in as {0}".format(login_username))
            else:
                self.logger.info("Failed to login as {0}".format(login_username))

        match = _url_re.match(self.url)
        username = match.group("username")

        channel = self._get_channel_info(username)
        if channel.get("BPWD") == "Y":
            self.logger.error("Stream is Password-Protected")
            return
        elif channel.get("RESULT") == -6:
            self.logger.error("Login required")
            return
        elif channel.get("RESULT") != CHANNEL_RESULT_OK:
            return

        (broadcast, rmd, cdn) = (channel["BNO"], channel["RMD"], channel["CDN"])
        if not (broadcast and rmd and cdn):
            return

        for qkey in QUALITYS:
            hls_stream = self._get_hls_stream(broadcast, username, qkey, cdn, rmd)
            if hls_stream:
                yield qkey, hls_stream


__plugin__ = AfreecaTV



import logging
import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream
from streamlink.utils import parse_json

log = logging.getLogger(__name__)


class NBCNews(Plugin):
    url_re = re.compile(r'https?://(?:www\.)?nbcnews\.com/now')
    js_re = re.compile(r'https://ndassets\.s-nbcnews\.com/main-[0-9a-f]{20}\.js')
    api_re = re.compile(r'NEWS_NOW_PID="([0-9]+)"')
    api_url = 'https://stream.nbcnews.com/data/live_sources_{0}.json'
    api_schema = validate.Schema(validate.transform(parse_json), {
        'videoSources': [{
            'sourceUrl': validate.url(),
            'type': validate.text
        }]
    }, validate.get('videoSources'), validate.get(0))

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def get_title(self):
        return 'NBC News Now'

    def _get_streams(self):
        html = self.session.http.get(self.url).text
        match = self.js_re.search(html)
        js = self.session.http.get(match.group(0)).text
        match = self.api_re.search(js)
        log.debug('API ID: {0}'.format(match.group(1)))
        api_url = self.api_url.format(match.group(1))
        stream = self.session.http.get(api_url, schema=self.api_schema)
        log.trace('{0!r}'.format(stream))
        if stream['type'].lower() != 'live':
            log.error('invalid stream type "{0}"'.format(stream['type']))
            return
        return HLSStream.parse_variant_playlist(self.session, stream['sourceUrl'])


__plugin__ = NBCNews



# -*- coding: utf-8 -*-
import unittest

from streamlink.compat import is_py3, is_py2
from streamlink.utils.encoding import maybe_decode, maybe_encode


class TestUtilsEncoding(unittest.TestCase):

    @unittest.skipUnless(is_py2, "only applicable in Python 2")
    def test_maybe_encode_py2(self):
        self.assertEqual(maybe_encode(u"test \u07f7"), "test \xdf\xb7")

    @unittest.skipUnless(is_py2, "only applicable in Python 2")
    def test_maybe_decode_py2(self):
        self.assertEqual(maybe_decode("test \xdf\xb7"), u"test \u07f7")

    @unittest.skipUnless(is_py3, "only applicable in Python 3")
    def test_maybe_encode_py3(self):
        self.assertEqual(maybe_encode(u"test \u07f7"), u"test \u07f7")

    @unittest.skipUnless(is_py3, "only applicable in Python 3")
    def test_maybe_decode_py3(self):
        self.assertEqual(maybe_decode(u"test \u07f7"), u"test \u07f7")



import re

from itertools import chain

from streamlink.compat import urlparse
from streamlink.plugin import Plugin
from streamlink.plugin.api import StreamMapper, validate
from streamlink.stream import HLSStream, HTTPStream, RTMPStream
from streamlink.utils import absolute_url

HLS_PLAYLIST_BASE = "http://www.smashcast.tv{0}"
LIVE_API = "http://www.smashcast.tv/api/media/live/{0}?showHidden=true&liveonly=false"
PLAYER_API = "http://www.smashcast.tv/api/player/config/{0}/{1}?embed=false&showHidden=true"
SWF_BASE = "http://edge.vie.hitbox.tv/static/player/flowplayer/"
SWF_URL = SWF_BASE + "flowplayer.commercial-3.2.16.swf"
VOD_BASE_URL = "http://www.smashcast.tv/"

_quality_re = re.compile(r"(\d+p)$")
_url_re = re.compile(r"""
    http(s)?://(www\.)?(hitbox|smashcast).tv
    /(?P<channel>[^/]+)
    (?:
        (?:/videos)?/(?P<media_id>[^/]+)
    )?
""", re.VERBOSE)

_live_schema = validate.Schema(
    {
        "livestream": [{
            "media_user_name": validate.text,
            validate.optional("media_hosted_media"): object,
            "media_is_live": validate.all(
                validate.text,
                validate.transform(int),
                validate.transform(bool)
            ),
            "media_id": validate.text
        }],
    },
    validate.get("livestream"),
    validate.length(1),
    validate.get(0)
)
_player_schema = validate.Schema(
    {
        "clip": {
            "baseUrl": validate.any(None, validate.text),
            "bitrates": validate.all(
                validate.filter(lambda b: b.get("url") and b.get("label")),
                [{
                    "label": validate.text,
                    "url": validate.text,
                }],
            )
        },
        validate.optional("playlist"): [{
            validate.optional("connectionProvider"): validate.text,
            validate.optional("netConnectionUrl"): validate.text,
            validate.optional("bitrates"): [{
                "label": validate.text,
                "url": validate.text,
                "provider": validate.text
            }]
        }],
        "plugins": validate.all(
            dict,
            validate.filter(lambda k, v: k in ["rtmp", "rtmpHitbox", "hls"]),
            {
                validate.text: {
                    validate.optional("netConnectionUrl"): validate.text,
                    "url": validate.text
                }
            }
        )
    }
)


class Hitbox(Plugin):
    @classmethod
    def can_handle_url(cls, url):
        return _url_re.match(url)

    def _get_quality(self, label):
        match = _quality_re.search(label)
        if match:
            return match.group(1)

        return "live"

    def _create_hls_streams(self, bitrate):
        url = bitrate["url"]
        quality = self._get_quality(bitrate["label"])

        if not url.startswith("http"):
            url = HLS_PLAYLIST_BASE.format(url)

        if bitrate["label"] == "Auto":
            try:
                streams = HLSStream.parse_variant_playlist(self.session, url)
                return streams.items()
            except IOError as err:
                self.logger.warning("Failed to extract HLS streams: {0}", err)
        else:
            return quality, HLSStream(self.session, url)

    def _create_rtmp_stream(self, rtmp, swf_url, bitrate):
        quality = self._get_quality(bitrate["label"])
        url = bitrate["url"]
        stream = RTMPStream(self.session, {
            "rtmp": rtmp,
            "pageUrl": self.url,
            "playpath": url,
            "swfVfy": swf_url,
            "live": True
        })

        return quality, stream

    def _get_live_streams(self, player):
        mappers = []
        swf_url = SWF_URL
        for playlist in player.get("playlist", []):
            bitrates = playlist.get("bitrates")
            provider = playlist.get("connectionProvider")
            rtmp = None

            if bitrates:
                rtmp = playlist.get("netConnectionUrl")
            elif provider and provider in player["plugins"]:
                provider = player["plugins"][provider]
                swf_name = provider["url"]
                swf_url = SWF_BASE + swf_name
                rtmp = provider["netConnectionUrl"]
                bitrates = player["clip"]["bitrates"]
            else:
                continue

            mapper = StreamMapper(
                cmp=lambda provider, bitrate: bitrate["provider"].startswith(provider)
            )
            mapper.map("hls", self._create_hls_streams)
            mapper.map("rtmp", self._create_rtmp_stream, rtmp, swf_url)
            mappers.append(mapper(bitrates))

        return chain.from_iterable(mappers)

    def _create_video_stream(self, cls, base_url, bitrate):
        url = absolute_url(base_url, bitrate["url"])
        if bitrate["label"].lower() == "auto":
            try:
                return cls.parse_variant_playlist(self.session, url).items()
            except IOError as err:
                self.logger.warning("Failed to extract HLS streams: {0}", err)
                return

        quality = self._get_quality(bitrate["label"])
        return quality, cls(self.session, url)

    def _get_video_streams(self, player):
        base_url = player["clip"]["baseUrl"] or VOD_BASE_URL
        mapper = StreamMapper(
            cmp=lambda ext, bitrate: urlparse(bitrate["url"]).path.endswith(ext)
        )
        mapper.map(".m3u8", self._create_video_stream, HLSStream, base_url)
        mapper.map(".mp4", self._create_video_stream, HTTPStream, base_url)
        mapper.map(".flv", self._create_video_stream, HTTPStream, base_url)

        return mapper(player["clip"]["bitrates"])

    def _get_streams(self):
        match = _url_re.match(self.url)
        if not match:
            return

        channel, media_id = match.group("channel", "media_id")
        self.logger.debug("Matched URL: channel={0}, media_id={1}".format(channel, media_id))
        if not media_id:
            res = self.session.http.get(LIVE_API.format(channel))
            livestream = self.session.http.json(res, schema=_live_schema)
            if livestream.get("media_hosted_media"):
                hosted = _live_schema.validate(livestream["media_hosted_media"])
                self.logger.info("{0} is hosting {1}", livestream["media_user_name"], hosted["media_user_name"])
                livestream = hosted

            if not livestream["media_is_live"]:
                return

            media_id = livestream["media_id"]
            media_type = "live"
        else:
            media_type = "video"

        res = self.session.http.get(PLAYER_API.format(media_type, media_id))
        player = self.session.http.json(res, schema=_player_schema)

        if media_type == "live":
            return self._get_live_streams(player)
        else:
            return self._get_video_streams(player)


__plugin__ = Hitbox



import unittest

from streamlink.plugins.nos import NOS


class TestPluginNOS(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'https://nos.nl/livestream/2220100-wk-sprint-schaatsen-1-000-meter-mannen.html',
        ]
        for url in should_match:
            self.assertTrue(NOS.can_handle_url(url))

    def test_can_handle_url_negative(self):
        should_not_match = [
            'https://example.com/index.html',
        ]
        for url in should_not_match:
            self.assertFalse(NOS.can_handle_url(url))



import unittest

from streamlink.plugins.streamme import StreamMe


class TestPluginStreamMe(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(StreamMe.can_handle_url("http://www.stream.me/nameofstream"))
        self.assertTrue(StreamMe.can_handle_url("https://stream.me/nameofstream"))

        # shouldn't match
        self.assertFalse(StreamMe.can_handle_url("http://www.livestream.me/nameofstream"))
        self.assertFalse(StreamMe.can_handle_url("http://www.streamme.com/nameofstream"))
        self.assertFalse(StreamMe.can_handle_url("http://www.streamme.me/nameofstream"))
        self.assertFalse(StreamMe.can_handle_url("http://www.youtube.com/"))



import unittest

from streamlink.plugins.tvibo import Tvibo


class TestPluginTvibo(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://player.tvibo.com/aztv/5929820',
            'http://player.tvibo.com/aztv/6858270/',
            'http://player.tvibo.com/aztv/3977238/',
        ]
        for url in should_match:
            self.assertTrue(Tvibo.can_handle_url(url))

    def test_can_handle_url_negative(self):
        should_not_match = [
            'http://www.idmantv.az/',
            'https://www.twitch.tv/twitch'
        ]
        for url in should_not_match:
            self.assertFalse(Tvibo.can_handle_url(url))



import unittest

from streamlink.plugins.atresplayer import AtresPlayer


class TestPluginAtresPlayer(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://www.atresplayer.com/directos/antena3/',
            'http://www.atresplayer.com/directos/lasexta/',
            'https://www.atresplayer.com/directos/antena3/',
            'https://www.atresplayer.com/flooxer/programas/unas/temporada-1/dario-eme-hache-sindy-takanashi-entrevista_123/',
        ]
        for url in should_match:
            self.assertTrue(AtresPlayer.can_handle_url(url))

    def test_can_handle_url_negative(self):
        should_not_match = [
            'http://www.example.com/',
        ]
        for url in should_not_match:
            self.assertFalse(AtresPlayer.can_handle_url(url))



import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream
from streamlink import NoStreamsError


class CubeTV(Plugin):
    _url_re = re.compile(r"https?://(www\.)?cube\.tv/(?P<channel>[^/]{2,})")

    _channel_info_api_url_base = "https://www.cube.tv/studio/info?cube_id={channel}"
    _stream_data_api_url_base = "https://www.cube.tv/studioApi/getStudioSrcBySid?sid={gid}&videoType=1&https=1"

    _channel_info_schema = validate.Schema({
        u"code": 1,
        u"msg": u"success",
        u"data": {
            u"gid": validate.text,
            u"cube_id": validate.text
        }
    })

    _stream_data_schema = validate.Schema({
        u"code": 1,
        u"msg": u"success",
        u"data": {
            u"video": u"hls",
            u"video_src": validate.url()
        }
    })

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def _get_api_res(self, user_id):
        try:
            res = self.session.http.get(self._channel_info_api_url_base.format(channel=user_id))
            return res
        except Exception:
            raise NoStreamsError(self.url)

    def _get_streams(self):
        user_id = self._url_re.match(self.url).group(2)
        res = self._get_api_res(user_id)
        user_gid = self.session.http.json(res, schema=self._channel_info_schema)['data']['gid']

        try:
            stream_data = self.session.http.get(self._stream_data_api_url_base.format(gid=user_gid))
            hls = self.session.http.json(stream_data, schema=self._stream_data_schema)['data']['video_src']
        except Exception:
            raise NoStreamsError(self.url)

        return HLSStream.parse_variant_playlist(self.session, hls)


__plugin__ = CubeTV



import unittest

from streamlink.plugins.btsports import BTSports


class TestPluginBTSports(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(BTSports.can_handle_url("https://sport.bt.com/btsportsplayer/football-match-1"))
        self.assertTrue(BTSports.can_handle_url("https://sport.bt.com/ss/Satellite/btsportsplayer/football-match-1"))

        # shouldn't match
        self.assertFalse(BTSports.can_handle_url("http://www.bt.com/"))
        self.assertFalse(BTSports.can_handle_url("http://bt.com/"))
        self.assertFalse(BTSports.can_handle_url("http://www.tvcatchup.com/"))
        self.assertFalse(BTSports.can_handle_url("http://www.youtube.com/"))



import logging
import re

from streamlink.exceptions import PluginError
from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents, validate
from streamlink.plugin.api.utils import parse_json
from streamlink.stream import HLSStream

log = logging.getLogger(__name__)


class StreamMe(Plugin):
    _url_re = re.compile(r'^https?://(?:www\.)?stream\.me/(\w+).*$')

    _streams_schema = validate.Schema(validate.all(
        validate.transform(parse_json),
        validate.any(
            validate.all({
                '_embedded': {
                    'streams': [{
                        'manifest': validate.url(),
                        'active': bool,
                        validate.optional('title'): validate.text,
                        validate.optional('privateLocked'): bool,
                        validate.optional('userSlug'): validate.text,
                    }]
                }},
                validate.get('_embedded'),
                validate.get('streams'),
                validate.get(0),
            ),
            {
                'reasons': [
                    {
                        'message': validate.text,
                        validate.optional('code'): validate.text,
                    }
                ]
            }
        )
    ))
    _formats_schema = validate.Schema(validate.all(
        validate.transform(parse_json),
        {
            'formats': {
                'mp4-hls': {
                    'encodings': [{
                        'videoHeight': int,
                        'location': validate.url()
                    }],
                    validate.optional('origin'): validate.any(None, {
                        validate.optional('location'): validate.url(),
                    })
                }
            }
        },
        validate.get('formats'),
        validate.get('mp4-hls'),
    ))

    API_CHANNEL = 'https://www.stream.me/api-user/v1/{0}/channel'
    STREAM_WEIGHTS = {
        'source': 65535,
    }

    def __init__(self, url):
        super(StreamMe, self).__init__(url)
        self.author = None
        self.title = None
        self.session.http.headers.update({'User-Agent': useragents.FIREFOX})

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    @classmethod
    def stream_weight(cls, stream):
        if stream in cls.STREAM_WEIGHTS:
            return cls.STREAM_WEIGHTS[stream], 'streamme'
        return Plugin.stream_weight(stream)

    def get_author(self):
        if self.author is None:
            self._get_channel_data()
        return self.author

    def get_title(self):
        if self.title is None:
            self._get_channel_data()
        return self.title

    def _get_channel_data(self):
        username = self._url_re.match(self.url).group(1)
        data = self.session.http.get(self.API_CHANNEL.format(username),
                                     acceptable_status=(200, 403, 404),
                                     schema=self._streams_schema)
        if data.get('reasons'):
            raise PluginError(data['reasons'][0]['message'])
        self.author = data.get('userSlug')
        self.title = data.get('title')
        return data

    def _get_streams(self):
        data = self._get_channel_data()
        log.trace('{0!r}'.format(data))
        if not data['active']:
            log.error('Stream is not active')
            return

        hls_streams = self.session.http.get(data['manifest'],
                                            schema=self._formats_schema)
        for s in hls_streams['encodings']:
            log.trace('{0!r}'.format(s))
            yield ('{0}p'.format(s['videoHeight']),
                   HLSStream(self.session, s['location']))

        source = hls_streams.get('origin')
        if source:
            log.trace('{0!r}'.format(source))
            yield 'source', HLSStream(self.session, source['location'])


__plugin__ = StreamMe



from __future__ import print_function
import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream


class RaiPlay(Plugin):
    url_re = re.compile(r"https?://(?:www\.)?raiplay\.it/dirette/(\w+)/?")
    stream_re = re.compile(r"data-video-url.*?=.*?\"([^\"]+)\"")
    stream_schema = validate.Schema(
        validate.all(
            validate.transform(stream_re.search),
            validate.any(
                None,
                validate.all(validate.get(1), validate.url())
            )
        )
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_streams(self):
        channel = self.url_re.match(self.url).group(1)
        self.logger.debug("Found channel: {0}", channel)
        stream_url = self.session.http.get(self.url, schema=self.stream_schema)
        if stream_url:
            return HLSStream.parse_variant_playlist(self.session, stream_url, headers={"User-Agent": useragents.CHROME})


__plugin__ = RaiPlay



import unittest

from streamlink.plugins.mitele import Mitele


class TestPluginMitele(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "http://www.mitele.es/directo/bemad",
            "http://www.mitele.es/directo/boing",
            "http://www.mitele.es/directo/cuatro",
            "http://www.mitele.es/directo/divinity",
            "http://www.mitele.es/directo/energy",
            "http://www.mitele.es/directo/fdf",
            "http://www.mitele.es/directo/telecinco",
            "https://www.mitele.es/directo/gh-duo-24h-senal-1",
            "https://www.mitele.es/directo/gh-duo-24h-senal-2",
        ]
        for url in should_match:
            self.assertTrue(Mitele.can_handle_url(url))

        should_not_match = [
            "http://www.mitele.es",
        ]
        for url in should_not_match:
            self.assertFalse(Mitele.can_handle_url(url))



import unittest

from streamlink.plugins.lrt import LRT


class TestPluginLRT(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "https://www.lrt.lt/mediateka/tiesiogiai/lrt-opus",
            "https://www.lrt.lt/mediateka/tiesiogiai/lrt-klasika",
            "https://www.lrt.lt/mediateka/tiesiogiai/lrt-radijas",
            "https://www.lrt.lt/mediateka/tiesiogiai/lrt-lituanica",
            "https://www.lrt.lt/mediateka/tiesiogiai/lrt-plius",
            "https://www.lrt.lt/mediateka/tiesiogiai/lrt-televizija",
        ]
        for url in should_match:
            self.assertTrue(LRT.can_handle_url(url), url)

    def test_can_handle_url_negative(self):
        should_not_match = [
            "https://www.lrt.lt",
            "https://www.youtube.com",
            "https://www.lrt.lt/mediateka/irasas/1013694276/savanoriai-tures-galimybe-pamatyti-popieziu-is-arciau",

        ]
        for url in should_not_match:
            self.assertFalse(LRT.can_handle_url(url), url)



from ..exceptions import StreamError
from streamlink.stream.stream import Stream

from streamlink.stream.akamaihd import AkamaiHDStream
from streamlink.stream.hds import HDSStream
from streamlink.stream.hls import HLSStream
from streamlink.stream.http import HTTPStream
from streamlink.stream.rtmpdump import RTMPStream
from streamlink.stream.dash import DASHStream
from streamlink.stream.streamprocess import StreamProcess
from streamlink.stream.wrappers import StreamIOWrapper, StreamIOIterWrapper, StreamIOThreadWrapper

from streamlink.stream.flvconcat import extract_flv_header_tags
from streamlink.stream.playlist import Playlist, FLVPlaylist



import unittest

from streamlink.plugins.live_russia_tv import LiveRussia


class TestPluginLiveRussiaTv(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "https://live.russia.tv/channel/1",
            "https://live.russia.tv/channel/199",
            "https://live.russia.tv/",
            "https://live.russia.tv/video/show/brand_id/60473/episode_id/2187772/video_id/2204594"
        ]
        for url in should_match:
            self.assertTrue(LiveRussia.can_handle_url(url))

        should_not_match = [
            "http://live.france.tv/somethingelse",
            "https://youtube.com/watch?v=4567uj"
        ]
        for url in should_not_match:
            self.assertFalse(LiveRussia.can_handle_url(url))



import json
import logging
import sys
from getpass import getpass

from streamlink.plugin.plugin import UserInputRequester
from .compat import input
from .utils import JSONEncoder

log = logging.getLogger("streamlink.cli")


class ConsoleUserInputRequester(UserInputRequester):
    """
    Request input from the user on the console using the standard ask/askpass methods
    """
    def __init__(self, console):
        self.console = console

    def ask(self, prompt):
        if sys.stdin.isatty():
            return self.console.ask(prompt.strip() + ": ")
        else:
            raise IOError("no TTY available")

    def ask_password(self, prompt):
        if sys.stdin.isatty():
            return self.console.askpass(prompt.strip() + ": ")
        else:
            raise IOError("no TTY available")


class ConsoleOutput(object):
    def __init__(self, output, streamlink, json=False):
        self.streamlink = streamlink

        self.json = json
        self.output = output

    def set_level(self, level):
        self.streamlink.set_loglevel(level)

    def set_output(self, output):
        self.output = output

    @classmethod
    def ask(cls, msg, *args, **kwargs):
        if sys.stdin.isatty():
            formatted = msg.format(*args, **kwargs)
            sys.stderr.write(formatted)

            try:
                answer = input()
            except Exception:
                answer = ""

            return answer.strip()
        else:
            return ""

    @classmethod
    def askpass(cls, msg, *args, **kwargs):
        if sys.stdin.isatty():
            return getpass(msg.format(*args, **kwargs))
        else:
            return ""

    def msg(self, msg, *args, **kwargs):
        formatted = msg.format(*args, **kwargs)
        formatted = u"{0}\n".format(formatted)

        self.output.write(formatted)

    def msg_json(self, obj):
        if not self.json:
            return

        if hasattr(obj, "__json__"):
            obj = obj.__json__()

        msg = json.dumps(obj, cls=JSONEncoder,
                         indent=2)
        self.msg(u"{0}", msg)

        if isinstance(obj, dict) and obj.get("error"):
            sys.exit(1)

    def exit(self, msg, *args, **kwargs):
        formatted = msg.format(*args, **kwargs)

        if self.json:
            obj = dict(error=formatted)
            self.msg_json(obj)
        else:
            msg = u"error: {0}".format(formatted)
            self.msg(u"{0}", msg)

        sys.exit(1)


__all__ = ["ConsoleOutput"]



import unittest

from streamlink.plugins.clubbingtv import ClubbingTV


class TestPluginClubbingTV(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(ClubbingTV.can_handle_url("https://www.clubbingtv.com/live"))
        self.assertTrue(ClubbingTV.can_handle_url("https://www.clubbingtv.com/video/play/3950/moonlight/"))
        self.assertTrue(ClubbingTV.can_handle_url("https://www.clubbingtv.com/video/play/2897"))
        self.assertTrue(ClubbingTV.can_handle_url("https://www.clubbingtv.com/tomer-s-pick/"))

        # shouldn't match
        self.assertFalse(ClubbingTV.can_handle_url("http://www.tvcatchup.com/"))
        self.assertFalse(ClubbingTV.can_handle_url("http://www.youtube.com/"))



import unittest

from streamlink.plugins.facebook import Facebook


class TestPluginFacebook(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(Facebook.can_handle_url("https://www.facebook.com/nos/videos/1725546430794241/"))
        self.assertTrue(Facebook.can_handle_url("https://www.facebook.com/nytfood/videos/1485091228202006/"))
        self.assertTrue(Facebook.can_handle_url("https://www.facebook.com/SporTurkTR/videos/798553173631138/"))
        self.assertTrue(Facebook.can_handle_url("https://www.facebook.com/119555411802156/posts/500665313691162/"))
        self.assertTrue(Facebook.can_handle_url("https://www.facebookcorewwwi.onion/SporTurkTR/videos/798553173631138/"))

        # shouldn't match
        self.assertFalse(Facebook.can_handle_url("https://www.facebook.com"))



import re

from streamlink.compat import quote
from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents, validate
from streamlink.plugin.api.utils import itertags
from streamlink.stream import HLSStream
from streamlink.utils import parse_json


def get_json(text):
    for script in itertags(text, "script"):
        if script.attributes.get("type") == "application/json" \
                and script.attributes.get("data-content") == "VideoPlayer.Config":
            return script.text
    return None


class Welt(Plugin):
    _re_url = re.compile(
        r"""https?://(\w+\.)?welt\.de/?""",
        re.IGNORECASE
    )
    _re_url_vod = re.compile(
        r"""mediathek""",
        re.IGNORECASE
    )
    _url_vod = "https://www.welt.de/onward/video/play/{0}"
    _schema = validate.Schema(
        validate.transform(get_json),
        validate.transform(parse_json),
        validate.get("sources"),
        validate.filter(lambda obj: obj["extension"] == "m3u8"),
        validate.map(lambda obj: obj["src"]),
        validate.get(0)
    )

    @classmethod
    def can_handle_url(cls, url):
        return cls._re_url.match(url) is not None

    def __init__(self, url):
        Plugin.__init__(self, url)
        self.url = url
        self.isVod = self._re_url_vod.search(url) is not None

    def _get_streams(self):
        headers = {"User-Agent": useragents.CHROME}
        hls_url = self.session.http.get(self.url, headers=headers, schema=self._schema)
        headers["Referer"] = self.url

        if self.isVod:
            url = self._url_vod.format(quote(hls_url, safe=""))
            hls_url = self.session.http.get(url, headers=headers).url

        return HLSStream.parse_variant_playlist(self.session, hls_url, headers=headers)


__plugin__ = Welt



import unittest

from streamlink.plugins.swisstxt import Swisstxt


class TestPluginSwisstxt(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "http://www.srf.ch/sport/resultcenter/tennis?eventId=338052",
            "http://live.rsi.ch/tennis.html?eventId=338052",
            "http://live.rsi.ch/sport.html?eventId=12345"
        ]
        for url in should_match:
            self.assertTrue(Swisstxt.can_handle_url(url))

        should_not_match = [
            # regular srgssr sites
            "http://srf.ch/play/tv/live",
            "http://www.rsi.ch/play/tv/live#?tvLiveId=livestream_La1",
            "http://rsi.ch/play/tv/live?tvLiveId=livestream_La1",
            "http://www.rtr.ch/play/tv/live",
            "http://rtr.ch/play/tv/live",
            "http://rts.ch/play/tv/direct#?tvLiveId=3608506",
            "http://www.srf.ch/play/tv/live#?tvLiveId=c49c1d64-9f60-0001-1c36-43c288c01a10",
            "http://www.rts.ch/sport/direct/8328501-tennis-open-daustralie.html",
            "http://www.rts.ch/play/tv/tennis/video/tennis-open-daustralie?id=8328501",
            # other sites
            "http://www.crunchyroll.com/gintama",
            "http://www.crunchyroll.es/gintama",
            "http://www.youtube.com/"
        ]
        for url in should_not_match:
            self.assertFalse(Swisstxt.can_handle_url(url))



import unittest

from streamlink.plugins.telefe import Telefe


class TestPluginTelefe(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(Telefe.can_handle_url("http://telefe.com/pone-a-francella/temporada-1/programa-01/"))
        self.assertTrue(Telefe.can_handle_url("http://telefe.com/los-simuladores/temporada-1/capitulo-01/"))
        self.assertTrue(Telefe.can_handle_url("http://telefe.com/dulce-amor/capitulos/capitulo-01/"))

        # shouldn't match
        self.assertFalse(Telefe.can_handle_url("http://telefe.com/"))
        self.assertFalse(Telefe.can_handle_url("http://www.telefeinternacional.com.ar/"))
        self.assertFalse(Telefe.can_handle_url("http://marketing.telefe.com/"))



import re

from streamlink.plugin import Plugin
from streamlink.plugin.api import useragents
from streamlink.stream import HLSStream
from streamlink.utils import parse_json


class ElTreceTV(Plugin):
    _url_re = re.compile(r'https?://(?:www\.)?eltrecetv.com.ar/.+')

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url)

    def _get_streams(self):
        if "eltrecetv.com.ar/vivo" in self.url.lower():
            try:
                self.session.http.headers = {
                    'Referer': self.url,
                    'User-Agent': useragents.ANDROID
                }
                res = self.session.http.get('https://api.iamat.com/metadata/atcodes/eltrece')
                yt_id = parse_json(res.text)["atcodes"][0]["context"]["ahora"]["vivo"]["youtubeVideo"]
                yt_url = "https://www.youtube.com/watch?v={0}".format(yt_id)
                return self.session.streams(yt_url)
            except BaseException:
                self.logger.info("Live content is temporarily unavailable. Please try again later.")
        else:
            try:
                self.session.http.headers = {
                    'Referer': self.url,
                    'User-Agent': useragents.CHROME
                }
                res = self.session.http.get(self.url)
                _player_re = re.compile(r'''data-kaltura="([^"]+)"''')
                match = _player_re.search(res.text)
                if not match:
                    return
                entry_id = parse_json(match.group(1).replace("&quot;", '"'))["entryId"]
                hls_url = "https://vodgc.com/p/111/sp/11100/playManifest/entryId/" \
                          "{0}/format/applehttp/protocol/https/a.m3u8".format(entry_id)
                return HLSStream.parse_variant_playlist(self.session, hls_url)
            except BaseException:
                self.logger.error("The requested VOD content is unavailable.")


__plugin__ = ElTreceTV



import re
import logging

from streamlink.plugin import Plugin
from streamlink.plugin.api.utils import itertags
from streamlink.plugins.youtube import YouTube
from streamlink.stream import HLSStream
from streamlink.utils import update_scheme

log = logging.getLogger(__name__)


class Dogus(Plugin):
    """
    Support for live streams from Dogus sites include ntv, ntvspor, and kralmuzik
    """

    url_re = re.compile(r"""https?://(?:www.)?
        (?:
            ntv.com.tr/canli-yayin/ntv|
            ntvspor.net/canli-yayin|
            kralmuzik.com.tr/tv/|
            eurostartv.com.tr/canli-izle
        )/?""", re.VERBOSE)
    mobile_url_re = re.compile(r"""(?P<q>["'])(?P<url>(https?:)?//[^'"]*?/live/hls/[^'"]*?\?token=)
                                   (?P<token>[^'"]*?)(?P=q)""", re.VERBOSE)
    token_re = re.compile(r"""token=(?P<q>["'])(?P<token>[^'"]*?)(?P=q)""")
    kral_token_url = "https://service.kralmuzik.com.tr/version/gettoken"

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def _get_streams(self):
        res = self.session.http.get(self.url)

        # Look for Youtube embedded video first
        for iframe in itertags(res.text, 'iframe'):
            if YouTube.can_handle_url(iframe.attributes.get("src")):
                log.debug("Handing off to YouTube plugin")
                return self.session.streams(iframe.attributes.get("src"))

        # Next check for HLS URL with token
        mobile_url_m = self.mobile_url_re.search(res.text)
        mobile_url = mobile_url_m and update_scheme(self.url, mobile_url_m.group("url"))
        if mobile_url:
            log.debug("Found mobile stream: {0}".format(mobile_url_m.group(0)))

            token = mobile_url_m and mobile_url_m.group("token")
            if not token and "kralmuzik" in self.url:
                log.debug("Getting Kral Muzik HLS stream token from API")
                token = self.session.http.get(self.kral_token_url).text
            elif not token:
                # if no token is in the url, try to find it else where in the page
                log.debug("Searching for HLS stream token in URL")
                token_m = self.token_re.search(res.text)
                token = token_m and token_m.group("token")

            return HLSStream.parse_variant_playlist(self.session, mobile_url + token,
                                                    headers={"Referer": self.url})


__plugin__ = Dogus



import unittest

from streamlink.plugins.raiplay import RaiPlay


class TestPluginRaiPlay(unittest.TestCase):
    def test_can_handle_url(self):
        # should match
        self.assertTrue(RaiPlay.can_handle_url("http://www.raiplay.it/dirette/rai1"))
        self.assertTrue(RaiPlay.can_handle_url("http://www.raiplay.it/dirette/rai2"))
        self.assertTrue(RaiPlay.can_handle_url("http://www.raiplay.it/dirette/rai3"))
        self.assertTrue(RaiPlay.can_handle_url("http://raiplay.it/dirette/rai3"))
        self.assertTrue(RaiPlay.can_handle_url("https://raiplay.it/dirette/rai3"))
        self.assertTrue(RaiPlay.can_handle_url("http://www.raiplay.it/dirette/rainews24"))
        self.assertTrue(RaiPlay.can_handle_url("https://www.raiplay.it/dirette/rainews24"))

        # shouldn't match
        self.assertFalse(RaiPlay.can_handle_url("http://www.adultswim.com/videos/streams/toonami"))
        self.assertFalse(RaiPlay.can_handle_url("http://www.tvcatchup.com/"))
        self.assertFalse(RaiPlay.can_handle_url("http://www.youtube.com/"))



import unittest

from streamlink.plugins.zdf_mediathek import zdf_mediathek


class TestPluginzdf_mediathek(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://www.zdf.de/live-tv',
            'https://www.zdf.de/sender/zdf/zdf-live-beitrag-100.html',
            'https://www.zdf.de/sender/zdfneo/zdfneo-live-beitrag-100.html',
            'https://www.zdf.de/sender/3sat/3sat-live-beitrag-100.html',
            'https://www.zdf.de/sender/phoenix/phoenix-live-beitrag-100.html',
            'https://www.zdf.de/sender/arte/arte-livestream-100.html',
            'https://www.zdf.de/sender/kika/kika-live-beitrag-100.html',
            'https://www.zdf.de/dokumentation/zdfinfo-doku/zdfinfo-live-beitrag-100.html',
            'https://www.zdf.de/comedy/heute-show/videos/diy-hazel-habeck-100.html',
            'https://www.zdf.de/nachrichten/heute-sendungen/so-wird-das-wetter-102.html',
        ]
        for url in should_match:
            self.assertTrue(zdf_mediathek.can_handle_url(url))

    def test_can_handle_url_negative(self):
        should_not_match = [
            'https://www.example.com',
        ]
        for url in should_not_match:
            self.assertFalse(zdf_mediathek.can_handle_url(url))



import unittest  # noqa: F401

import pytest

from streamlink.plugins.albavision import Albavision


class TestPluginAlbavision:
    valid_urls = [
        ("https://www.elnueve.com.ar/en-vivo",),
        ("http://www.rts.com.ec/envivo",),
        ("https://www.tvc.com.ec/envivo",),
        ("http://www.atv.pe/envivo/ATV",),
        ("http://www.atv.pe/envivo/ATVMas",),
    ]
    invalid_urls = [
        ("https://news.now.com/home/local",),
        ("http://media.now.com.hk/",),
        ("https://www.youtube.com",)
    ]

    @pytest.mark.parametrize(["url"], valid_urls)
    def test_can_handle_url(self, url):
        assert Albavision.can_handle_url(url), "url should be handled"

    @pytest.mark.parametrize(["url"], invalid_urls)
    def test_can_handle_url_negative(self, url):
        assert not Albavision.can_handle_url(url), "url should not be handled"

    def test_transform(self):
        token = Albavision.transform_token(u'6b425761cc8a86569b1a05a9bf1870c95fca717dOK', 436171)
        assert token == "6b425761cc8a86569b1a05a9bf1870c95fca717d"



import unittest

from streamlink.plugins.nbcsports import NBCSports


class TestPluginNBCSports(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'https://www.nbcsports.com/video/evertons-wayne-rooney-completes-hat-trick-goal-midfield',
        ]
        for url in should_match:
            self.assertTrue(NBCSports.can_handle_url(url))

    def test_can_handle_url_negative(self):
        should_not_match = [
            'https://example.com/index.html',
        ]
        for url in should_not_match:
            self.assertFalse(NBCSports.can_handle_url(url))



import unittest

from streamlink.plugins.vinhlongtv import VinhLongTV


class TestPluginVinhLongTV(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://thvli.vn/live/thvl1-hd/aab94d1f-44e1-4992-8633-6d46da08db42',
            'http://thvli.vn/live/thvl2-hd/bc60bddb-99ac-416e-be26-eb4d0852f5cc',
            'http://thvli.vn/live/phat-thanh/c87174ba-7aeb-4cb4-af95-d59de715464c',
        ]
        for url in should_match:
            self.assertTrue(VinhLongTV.can_handle_url(url))

        should_not_match = [
            'https://example.com/index.html',
        ]
        for url in should_not_match:
            self.assertFalse(VinhLongTV.can_handle_url(url))



import logging
import re

from streamlink.plugin import Plugin
from streamlink.stream import HLSStream

log = logging.getLogger(__name__)


class Tvibo(Plugin):

    _url_re = re.compile(r"https?://player\.tvibo\.com/\w+/(?P<id>\d+)")
    _api_url = "http://panel.tvibo.com/api/player/streamurl/{id}"

    @classmethod
    def can_handle_url(cls, url):
        return cls._url_re.match(url) is not None

    def _get_streams(self):
        channel_id = self._url_re.match(self.url).group("id")

        api_response = self.session.http.get(
            self._api_url.format(id=channel_id),
            acceptable_status=(200, 404))

        data = self.session.http.json(api_response)
        log.trace("{0!r}".format(data))
        if data.get("st"):
            yield "source", HLSStream(self.session, data["st"])
        elif data.get("error"):
            log.error(data["error"]["message"])


__plugin__ = Tvibo



import re
import time
import json

from streamlink.compat import quote
from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HTTPStream, HLSStream

ROOM_API = "https://www.panda.tv/api_room_v3?token=&hostid={0}&roomid={1}&roomkey={2}&_={3}&param={4}&time={5}&sign={6}"
ROOM_API_V2 = "https://www.panda.tv/api_room_v2?roomid={0}&_={1}"
SD_URL_PATTERN = "https://pl{0}.live.panda.tv/live_panda/{1}.flv?sign={2}&ts={3}&rid={4}"
HD_URL_PATTERN = "https://pl{0}.live.panda.tv/live_panda/{1}_mid.flv?sign={2}&ts={3}&rid={4}"
OD_URL_PATTERN = "https://pl{0}.live.panda.tv/live_panda/{1}_small.flv?sign={2}&ts={3}&rid={4}"

_url_re = re.compile(r"http(s)?://(?P<prefix>\w+.)?panda.tv/(?P<channel>[^/&?]+)")
_room_id_re = re.compile(r'data-room-id="(\d+)"')
_status_re = re.compile(r'"status"\s*:\s*"(\d+)"\s*,\s*"display_type"')
_room_key_re = re.compile(r'"room_key"\s*:\s*"(.+?)"')
_hostid_re = re.compile(r'\\"hostid\\"\s*:\s*\\"(.+?)\\"')
_param_re = re.compile(r'"param"\s*:\s*"(.+?)"\s*,\s*"time"')
_time_re = re.compile(r'"time"\s*:\s*(\d+)')
_sign_re = re.compile(r'"sign"\s*:\s*"(.+?)"')
_sd_re = re.compile(r'"SD"\s*:\s*"(\d+)"')
_hd_re = re.compile(r'"HD"\s*:\s*"(\d+)"')
_od_re = re.compile(r'"OD"\s*:\s*"(\d+)"')
_roominfo_re = re.compile(r'window\.HOSTINFO=({.*});')

STREAM_WEIGHTS = {
    "source": 1080,
    "medium": 720,
    "low": 480
}

_room_schema = validate.Schema(
    {
        "data": validate.any(
            validate.text,
            dict,
            {
                "videoinfo": validate.any(
                    validate.text,
                    {
                        "plflag_list": validate.text,
                        "plflag": validate.text
                    }
                )
            }
        )
    },
    validate.get("data"))


class Pandatv(Plugin):
    @classmethod
    def can_handle_url(cls, url):
        return _url_re.match(url)

    @classmethod
    def stream_weight(cls, stream):
        if stream in STREAM_WEIGHTS:
            return STREAM_WEIGHTS[stream], "pandatv"
        return Plugin.stream_weight(stream)

    def _get_streams(self):
        match = _url_re.match(self.url)
        prefix = match.group("prefix")
        channel = match.group("channel")

        res = self.session.http.get(self.url)

        if prefix == 'xingyan.':
            roominfo = _roominfo_re.search(res.text).group(1)
            roominfo = json.loads(roominfo)
            videoinfo = roominfo['videoinfo']
            if videoinfo['hlsurl']:
                yield 'source', HLSStream(self.session, videoinfo['hlsurl'])

            # wangsu cdn prior to others, wangsu = 0, alicloud = 1, qcloud = videoinfo['streamurl']
            _cdn = 0
            if videoinfo['zl'][_cdn]['streamurl']:
                yield 'source', HTTPStream(self.session, videoinfo['zl'][_cdn]['streamurl'])
            if videoinfo['zl'][_cdn]['streamtrans']['mid']:
                yield 'medium', HTTPStream(self.session, videoinfo['zl'][_cdn]['streamtrans']['mid'])
            if videoinfo['zl'][_cdn]['streamtrans']['small']:
                yield 'low', HTTPStream(self.session, videoinfo['zl'][_cdn]['streamtrans']['small'])
            return

        try:
            channel = int(channel)
        except ValueError:
            channel = _room_id_re.search(res.text).group(1)

        ts = int(time.time())
        url = ROOM_API_V2.format(channel, ts)
        res = self.session.http.get(url)

        try:
            status = _status_re.search(res.text).group(1)
            room_key = _room_key_re.search(res.text).group(1)
            hostid = _hostid_re.search(res.text).group(1)
            param = _param_re.search(res.text).group(1)
            tt = _time_re.search(res.text).group(1)
            sign = _sign_re.search(res.text).group(1)
            sd = _sd_re.search(res.text).group(1)
            hd = _hd_re.search(res.text).group(1)
            od = _od_re.search(res.text).group(1)
        except AttributeError:
            self.logger.info("Not a valid room url.")
            return

        if status != '2':
            self.logger.info("Stream currently unavailable.")
            return

        ts = int(time.time())
        param = param.replace("\\", "")
        param = quote(param)
        url = ROOM_API.format(hostid, channel, room_key, ts, param, tt, sign)
        room = self.session.http.get(url)
        data = self.session.http.json(room, schema=_room_schema)
        if not isinstance(data, dict):
            self.logger.info("Please Check PandaTV Room API")
            return

        videoinfo = data.get('videoinfo')
        plflag_list = videoinfo.get('plflag_list')
        if not videoinfo or not plflag_list:
            self.logger.info("Please Check PandaTV Room API")
            return

        plflag = videoinfo.get('plflag')
        if not plflag or '_' not in plflag:
            self.logger.info("Please Check PandaTV Room API")
            return

        plflag_list = json.loads(plflag_list)
        backup = plflag_list["backup"]
        rid = plflag_list["auth"]["rid"]
        sign = plflag_list["auth"]["sign"]
        ts = plflag_list["auth"]["time"]

        backup.append(plflag)
        plflag0 = backup
        plflag0 = [i.split('_')[1] for i in plflag0]

        # let wangsu cdn priority, flag can see here in "H5PLAYER_CDN_LINES":
        # https://www.panda.tv/cmstatic/global-config.js
        lines = ["3", "4"]
        try:
            plflag1 = [i for i in plflag0 if i in lines][0]
        except IndexError:
            plflag1 = plflag0[0]

        if sd == '1':
            yield 'source', HTTPStream(self.session, SD_URL_PATTERN.format(plflag1, room_key, sign, ts, rid))

        if hd == '1':
            yield 'medium', HTTPStream(self.session, HD_URL_PATTERN.format(plflag1, room_key, sign, ts, rid))

        if od == '1':
            yield 'low', HTTPStream(self.session, OD_URL_PATTERN.format(plflag1, room_key, sign, ts, rid))

        return


__plugin__ = Pandatv



# -*- coding: utf-8 -*-
import unittest

from streamlink.plugins.deutschewelle import DeutscheWelle


class TestPluginDeutscheWelle(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "http://www.dw.com/en/media-center/live-tv/s-100825",
            "http://www.dw.com/fr/médiathèque/direct-tv/s-100948?channel=5",
            "http://www.dw.com/de/tim-bendzko-deutsch-pop-darling-auf-tour/av-39101077",
            "http://www.dw.com/de/trump-sagt-nein-die-welt-schüttelt-den-kopf/av-39096724",
            "http://www.dw.com/el/ο-τραμπ-θεωρεί-την-ευρώπη-ανταγωνιστή-όχι-εταίρο/av-39103742",
            "http://www.dw.com/ar/أي-البروتينات-صحية-الحيوانية-أم-النباتية/av-39095317",
        ]
        for url in should_match:
            self.assertTrue(DeutscheWelle.can_handle_url(url))

        should_not_match = [
            "http://www.tvcatchup.com/",
            "http://www.youtube.com/"
        ]
        for url in should_not_match:
            self.assertFalse(DeutscheWelle.can_handle_url(url))



import unittest

from streamlink.plugins.pluzz import Pluzz


class TestPluginPluzz(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "https://www.france.tv/france-2/direct.html",
            "https://www.france.tv/france-3/direct.html",
            "https://www.france.tv/france-3-franche-comte/direct.html",
            "https://www.france.tv/france-4/direct.html",
            "https://www.france.tv/france-5/direct.html",
            "https://www.france.tv/france-o/direct.html",
            "https://www.france.tv/franceinfo/direct.html",
            "https://www.france.tv/france-2/journal-20h00/141003-edition-du-lundi-8-mai-2017.html",
            "https://www.france.tv/france-o/underground/saison-1/132187-underground.html",
            "http://www.ludo.fr/heros/the-batman",
            "http://www.ludo.fr/heros/il-etait-une-fois-la-vie",
            "http://www.zouzous.fr/heros/oui-oui",
            "http://www.zouzous.fr/heros/marsupilami-1",
            "http://france3-regions.francetvinfo.fr/bourgogne-franche-comte/tv/direct/franche-comte",
            "http://sport.francetvinfo.fr/roland-garros/direct",
            "http://sport.francetvinfo.fr/roland-garros/live-court-3",
            "http://sport.francetvinfo.fr/roland-garros/andy-murray-gbr-1-andrey-kuznetsov-rus-1er-tour-court"
            + "-philippe-chatrier",
            "https://www.francetvinfo.fr/en-direct/tv.html"
        ]
        for url in should_match:
            self.assertTrue(Pluzz.can_handle_url(url))

        should_not_match = [
            "http://www.france.tv/",
            "http://pluzz.francetv.fr/",
            "http://www.ludo.fr/",
            "http://www.ludo.fr/jeux",
            "http://www.zouzous.fr/",
            "http://www.tvcatchup.com/",
            "http://www.youtube.com/"
        ]
        for url in should_not_match:
            self.assertFalse(Pluzz.can_handle_url(url))



from sys import getfilesystemencoding

from streamlink.compat import is_win32, is_py2


def get_filesystem_encoding():
    file_system_encoding = getfilesystemencoding()
    if file_system_encoding is None:  # `None` not possible after python 3.2
        if is_win32:
            file_system_encoding = 'mbcs'
        else:
            file_system_encoding = 'utf-8'
    return file_system_encoding


def maybe_encode(text, encoding="utf8"):
    if is_py2:
        if isinstance(text, unicode):
            return text.encode(encoding)
        else:
            return text
    else:
        return text


def maybe_decode(text, encoding="utf8"):
    if is_py2 and isinstance(text, str):
        return text.decode(encoding)
    else:
        return text


__all__ = ["get_filesystem_encoding", "maybe_decode", "maybe_encode"]



import unittest

from streamlink.plugins.delfi import Delfi


class TestPluginDelfi(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "http://www.delfi.lt/video/sportas/kroatiska-tvarka-kaliningrade-laisvai-liejosi-gerimai-dainos"
            + "-sokiai-ir-ziezirbos.d?id=78322857",
            "https://www.delfi.lt/video/sportas/zalgiris-atsidure-per-pergale-nuo-lkl-aukso.d?id=78321125",
            "https://www.delfi.lt/video/laidos/nba/warriors-cempioniskomis-tapusios-ketvirtos-finalo"
            + "-rungtynes.d?id=78246059",
            "http://rahvahaal.delfi.ee/news/videod/video-joviaalne-piduline-kaotab-raekoja-platsil-ilutulestiku"
            + "-ule-kontrolli-ja-raketid-lendavad-rahva-sekka?id=82681069",
            "http://www.delfi.lv/delfi-tv-ar-jani-domburu/pilnie-raidijumi/delfi-tv-ar-jani-domburu-atbild"
            + "-veselibas-ministre-anda-caksa-pilna-intervija?id=49515013"
        ]
        for url in should_match:
            self.assertTrue(Delfi.can_handle_url(url))

        should_not_match = [
            "http://www.tvcatchup.com/",
            "http://www.youtube.com/"
        ]
        for url in should_not_match:
            self.assertFalse(Delfi.can_handle_url(url))



import unittest
import os.path
from contextlib import contextmanager

from streamlink.plugin.plugin import UserInputRequester
from tests.mock import MagicMock, patch

from streamlink import Streamlink, PluginError
from streamlink_cli.console import ConsoleUserInputRequester
from tests.plugins.testplugin import TestPlugin as _TestPlugin


class TestPluginUserInput(unittest.TestCase):
    def setUp(self):
        self.session = Streamlink()

    @contextmanager
    def _mock_console_input(self, isatty=True):
        with patch('streamlink_cli.console.sys.stdin.isatty', return_value=isatty):
            mock_console = MagicMock()
            mock_console.ask.return_value = "username"
            mock_console.askpass.return_value = "password"
            yield ConsoleUserInputRequester(mock_console)

    def test_user_input_bad_class(self):
        p = _TestPlugin("http://example.com/stream")
        self.assertRaises(RuntimeError, p.bind, self.session, 'test_plugin', object())

    def test_user_input_not_implemented(self):
        p = _TestPlugin("http://example.com/stream")
        p.bind(self.session, 'test_plugin', UserInputRequester())
        self.assertRaises(PluginError, p.input_ask, 'test')
        self.assertRaises(PluginError, p.input_ask_password, 'test')

    def test_user_input_console(self):
        p = _TestPlugin("http://example.com/stream")
        with self._mock_console_input() as console_input:
            p.bind(self.session, 'test_plugin', console_input)
            self.assertEqual("username", p.input_ask("username"))
            self.assertEqual("password", p.input_ask_password("password"))
            console_input.console.ask.assert_called_with("username: ")
            console_input.console.askpass.assert_called_with("password: ")

    def test_user_input_console_no_tty(self):
        p = _TestPlugin("http://example.com/stream")
        with self._mock_console_input(isatty=False) as console_input:
            p.bind(self.session, 'test_plugin', console_input)
            self.assertRaises(PluginError, p.input_ask, "username")
            self.assertRaises(PluginError, p.input_ask_password, "password")

    def test_set_via_session(self):
        with self._mock_console_input() as console_input:
            session = Streamlink({"user-input-requester": console_input})
            session.load_plugins(os.path.join(os.path.dirname(__file__), "plugins"))

            p = session.resolve_url("http://test.se/channel")
            self.assertEqual("username", p.input_ask("username"))
            self.assertEqual("password", p.input_ask_password("password"))



from __future__ import print_function

import logging
import re
from functools import partial

from streamlink.plugin import Plugin
from streamlink.plugin.api import validate
from streamlink.stream import HLSStream, DASHStream
from streamlink.utils import parse_json, update_scheme, search_dict

log = logging.getLogger(__name__)


class AtresPlayer(Plugin):
    url_re = re.compile(r"https?://(?:www.)?atresplayer.com/")
    state_re = re.compile(r"""window.__PRELOADED_STATE__\s*=\s*({.*?});""", re.DOTALL)
    channel_id_schema = validate.Schema(
        validate.transform(state_re.search),
        validate.any(
            None,
            validate.all(
                validate.get(1),
                validate.transform(parse_json),
                validate.transform(partial(search_dict, key="href")),
            )
        )
    )
    player_api_schema = validate.Schema(
        validate.any(
            None,
            validate.all(
                validate.transform(parse_json),
                validate.transform(partial(search_dict, key="urlVideo")),
            )
        )
    )
    stream_schema = validate.Schema(
        validate.transform(parse_json),
        {"sources": [
            validate.all({
                "src": validate.url(),
                validate.optional("type"): validate.text
            })
        ]}, validate.get("sources"))

    @classmethod
    def can_handle_url(cls, url):
        return cls.url_re.match(url) is not None

    def __init__(self, url):
        # must be HTTPS
        super(AtresPlayer, self).__init__(update_scheme("https://", url))

    def _get_streams(self):
        api_urls = self.session.http.get(self.url, schema=self.channel_id_schema)
        _api_url = list(api_urls)[0]
        log.debug("API URL: {0}".format(_api_url))
        player_api_url = self.session.http.get(_api_url, schema=self.player_api_schema)
        for api_url in player_api_url:
            log.debug("Player API URL: {0}".format(api_url))
            for source in self.session.http.get(api_url, schema=self.stream_schema):
                log.debug("Stream source: {0} ({1})".format(source['src'], source.get("type", "n/a")))

                if "type" not in source or source["type"] == "application/vnd.apple.mpegurl":
                    streams = HLSStream.parse_variant_playlist(self.session, source["src"])
                    if not streams:
                        yield "live", HLSStream(self.session, source["src"])
                    else:
                        for s in streams.items():
                            yield s
                elif source["type"] == "application/dash+xml":
                    for s in DASHStream.parse_manifest(self.session, source["src"]).items():
                        yield s


__plugin__ = AtresPlayer



import unittest

from streamlink.plugins.tvp import TVP


class TestPluginTVP(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://tvpstream.vod.tvp.pl/?channel_id=14327511',
            'http://tvpstream.vod.tvp.pl/?channel_id=1455',
        ]
        for url in should_match:
            self.assertTrue(TVP.can_handle_url(url))

        should_not_match = [
            'http://tvp.pl/',
            'http://vod.tvp.pl/',
        ]
        for url in should_not_match:
            self.assertFalse(TVP.can_handle_url(url))



# -*- coding: utf-8 -*-
from streamlink_cli.utils.progress import terminal_width, get_cut_prefix
import unittest


class TestCliUtilProgess(unittest.TestCase):
    def test_terminal_width(self):
        self.assertEqual(10, terminal_width("ABCDEFGHIJ"))
        self.assertEqual(30, terminal_width("A你好世界こんにちは안녕하세요B"))
        self.assertEqual(30, terminal_width("·「」『』【】-=！@#￥%……&×（）"))
        pass

    def test_get_cut_prefix(self):
        self.assertEqual("녕하세요CD",
                         get_cut_prefix("你好世界こんにちは안녕하세요CD", 10))
        self.assertEqual("하세요CD",
                         get_cut_prefix("你好世界こんにちは안녕하세요CD", 9))
        self.assertEqual("こんにちは안녕하세요CD",
                         get_cut_prefix("你好世界こんにちは안녕하세요CD", 23))
        pass



import unittest

from streamlink.plugins.onetv import OneTV


class TestPluginOneTV(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            "https://www.1tv.ru/live",
            "http://www.1tv.ru/live",
            "http://www.1tv.ru/some-show/some-programme-2018-03-10",
            "https://www.ctc.ru/online",
            "http://www.ctc.ru/online",
            "https://www.chetv.ru/online",
            "http://www.chetv.ru/online",
            "https://www.ctclove.ru/online",
            "http://www.ctclove.ru/online",
            "https://www.domashny.ru/online"
            "http://www.domashny.ru/online"
        ]
        for url in should_match:
            self.assertTrue(OneTV.can_handle_url(url))

        should_not_match = [
            "https://www.youtube.com",
        ]
        for url in should_not_match:
            self.assertFalse(OneTV.can_handle_url(url))

    def test_channel(self):
        self.assertEqual(OneTV("http://1tv.ru/live").channel,
                         "1tv")
        self.assertEqual(OneTV("http://www.ctclove.ru/online").channel,
                         "ctc-love")
        self.assertEqual(OneTV("http://domashny.ru/online").channel,
                         "ctc-dom")

    def test_live_api_url(self):
        self.assertEqual(OneTV("http://1tv.ru/live").live_api_url,
                         "http://stream.1tv.ru/api/playlist/1tvch_as_array.json")

        self.assertEqual(OneTV("https://www.ctclove.ru/online").live_api_url,
                         "https://media.1tv.ru/api/v1/ctc/playlist/ctc-love_as_array.json")

        self.assertEqual(OneTV("http://domashny.ru/online").live_api_url,
                         "http://media.1tv.ru/api/v1/ctc/playlist/ctc-dom_as_array.json")



import unittest

from streamlink.plugins.huajiao import Huajiao


class TestPluginHuajiao(unittest.TestCase):
    def test_can_handle_url(self):
        should_match = [
            'http://www.huajiao.com/l/123123123',
        ]
        for url in should_match:
            self.assertTrue(Huajiao.can_handle_url(url))

    def test_can_handle_url_negative(self):
        should_not_match = [
            'https://example.com/index.html',
        ]
        for url in should_not_match:
            self.assertFalse(Huajiao.can_handle_url(url))



from .session import Streamlink


def streams(url, **params):
    """Attempts to find a plugin and extract streams from the *url*.

    *params* are passed to :func:`Plugin.streams`.

    Raises :exc:`NoPluginError` if no plugin is found.
    """

    session = Streamlink()
    return session.streams(url, **params)



import json
import logging
import re
import websocket
import threading
import time

from streamlink.plugin import Plugin, PluginArguments, PluginArgument
from streamlink.plugin.api import useragents
from streamlink.stream import HLSStream
from streamlink.compat import urlparse, unquote_plus


_log = logging.getLogger(__name__)

_url_re = re.compile(
    r"^https?://(?P<domain>live[0-9]*\.nicovideo\.jp)/watch/lv[0-9]*")

_login_url = "https://account.nicovideo.jp/login/redirector"

_login_url_params = {
    "show_button_twitter": 1,
    "show_button_facebook": 1,
    "next_url": "/"}


class NicoLive(Plugin):
    arguments = PluginArguments(
        PluginArgument(
            "email",
            argument_name="niconico-email",
            sensitive=True,
            metavar="EMAIL",
            help="The email or phone number associated with your "
                 "Niconico account"),
        PluginArgument(
            "password",
            argument_name="niconico-password",
            sensitive=True,
            metavar="PASSWORD",
            help="The password of your Niconico account"),
        PluginArgument(
            "user-session",
            argument_name="niconico-user-session",
            sensitive=True,
            metavar="VALUE",
            help="Value of the user-session token \n(can be used in "
                 "case you do not want to put your password here)"))

    is_stream_ready = False
    is_stream_ended = False
    watching_interval = 30
    watching_interval_worker_thread = None
    stream_reader = None
    _ws = None

    @classmethod
    def can_handle_url(cls, url):
        return _url_re.match(url) is not None

    def _get_streams(self):
        self.url = self.url.split("?")[0]
        self.session.http.headers.update({
            "User-Agent": useragents.CHROME,
        })

        if not self.get_wss_api_url():
            _log.debug("Coundn't extract wss_api_url. Attempting login...")
            if not self.niconico_web_login():
                return None
            if not self.get_wss_api_url():
                _log.error("Failed to get wss_api_url.")
                _log.error(
                    "Please check if the URL is correct, "
                    "and make sure your account has access to the video.")
                return None

        self.api_connect(self.wss_api_url)

        i = 0
        while not self.is_stream_ready:
            if i % 10 == 0:
                _log.debug("Waiting for permit...")
            if i == 600:
                _log.error("Waiting for permit timed out.")
                return None
            if self.is_stream_ended:
                return None
            time.sleep(0.1)
            i += 1

        streams = HLSStream.parse_variant_playlist(
            self.session, self.hls_stream_url)

        nico_streams = {}
        for s in streams:
            nico_stream = NicoHLSStream(streams[s], self)
            nico_streams[s] = nico_stream

        return nico_streams

    def get_wss_api_url(self):
        _log.debug("Getting video page: {0}".format(self.url))
        resp = self.session.http.get(self.url)

        try:
            self.wss_api_url = extract_text(
                resp.text, "&quot;webSocketUrl&quot;:&quot;", "&quot;")
        except Exception as e:
            _log.debug(e)
            _log.debug("Failed to extract wss api url")
            return False

        try:
            self.broadcast_id = extract_text(
                resp.text, "&quot;broadcastId&quot;:&quot;", "&quot;")
        except Exception as e:
            _log.debug(e)
            _log.warning("Failed to extract broadcast id")

        _log.debug("Video page response code: {0}".format(resp.status_code))
        _log.trace(u"Video page response body: {0}".format(resp.text))
        _log.debug("Got wss_api_url: {0}".format(self.wss_api_url))
        _log.debug("Got broadcast_id: {0}".format(self.broadcast_id))

        return self.wss_api_url.startswith("wss://")

    def api_on_open(self):
        self.send_playerversion()
        require_new_stream = not self.is_stream_ready
        self.send_getpermit(require_new_stream=require_new_stream)

    def api_on_error(self, ws, error=None):
        if error:
            _log.warning(error)
        _log.warning("wss api disconnected.")
        _log.warning("Attempting to reconnect in 5 secs...")
        time.sleep(5)
        self.api_connect(self.wss_api_url)

    def api_connect(self, url):
        # Proxy support adapted from the UStreamTV plugin (ustreamtv.py)
        proxy_url = self.session.get_option("https-proxy")
        if proxy_url is None:
            proxy_url = self.session.get_option("http-proxy")
        proxy_options = parse_proxy_url(proxy_url)
        if proxy_options.get('http_proxy_host'):
            _log.debug("Using proxy ({0}://{1}:{2})".format(
                proxy_options.get('proxy_type') or "http",
                proxy_options.get('http_proxy_host'),
                proxy_options.get('http_proxy_port') or 80))

        _log.debug("Connecting: {0}".format(url))
        self._ws = websocket.WebSocketApp(
            url,
            header=["User-Agent: {0}".format(useragents.CHROME)],
            on_open=self.api_on_open,
            on_message=self.handle_api_message,
            on_error=self.api_on_error)
        self.ws_worker_thread = threading.Thread(
            target=self._ws.run_forever,
            args=proxy_options)
        self.ws_worker_thread.daemon = True
        self.ws_worker_thread.start()

    def send_message(self, type_, body):
        msg = {"type": type_, "body": body}
        msg_json = json.dumps(msg)
        _log.debug(u"Sending: {0}".format(msg_json))
        if self._ws and self._ws.sock.connected:
            self._ws.send(msg_json)
        else:
            _log.warning("wss api is not connected.")

    def send_playerversion(self):
        body = {
            "command": "playerversion",
            "params": ["leo"]
        }
        self.send_message("watch", body)

    def send_getpermit(self, require_new_stream=True):
        body = {
            "command": "getpermit",
            "requirement": {
                "broadcastId": self.broadcast_id,
                "route": "",
                "stream": {
                    "protocol": "hls",
                    "requireNewStream": require_new_stream,
                    "priorStreamQuality": "abr",
                    "isLowLatency": True,
                    "isChasePlay": False
                },
                "room": {
                    "isCommentable": True,
                    "protocol": "webSocket"
                }
            }
        }
        self.send_message("watch", body)

    def send_watching(self):
        body = {
            "command": "watching",
            "params": [self.broadcast_id, "-1", "0"]
        }
        self.send_message("watch", body)

    def send_pong(self):
        self.send_message("pong", {})

    def handle_api_message(self, message):
        _log.debug(u"Received: {0}".format(message))
        message_parsed = json.loads(message)

        if message_parsed["type"] == "watch":
            body = message_parsed["body"]
            command = body["command"]

            if command == "currentstream":
                current_stream = body["currentStream"]
                self.hls_stream_url = current_stream["uri"]
                self.is_stream_ready = True

            elif command == "watchinginterval":
                self.watching_interval = int(body["params"][0])
                _log.debug("Got watching_interval: {0}".format(
                    self.watching_interval))

                if self.watching_interval_worker_thread is None:
                    _log.debug("send_watching_scheduler starting.")
                    self.watching_interval_worker_thread = threading.Thread(
                        target=self.send_watching_scheduler)
                    self.watching_interval_worker_thread.daemon = True
                    self.watching_interval_worker_thread.start()

                else:
                    _log.debug("send_watching_scheduler already running.")

            elif command == "disconnect":
                _log.info("Websocket API closed.")
                _log.info("Stream ended.")
                self.is_stream_ended = True

                if self.stream_reader is not None:
                    self.stream_reader.close()
                    _log.info("Stream reader closed.")

        elif message_parsed["type"] == "ping":
            self.send_pong()

    def send_watching_scheduler(self):
        """
        Periodically send "watching" command to the API.
        This is necessary to keep the session alive.
        """
        while not self.is_stream_ended:
            self.send_watching()
            time.sleep(self.watching_interval)

    def niconico_web_login(self):
        user_session = self.get_option("user-session")
        email = self.get_option("email")
        password = self.get_option("password")

        if user_session is not None:
            _log.info("User session cookie is provided. Using it.")
            self.session.http.cookies.set(
                "user_session",
                user_session,
                path="/",
                domain="nicovideo.jp")
            self.save_cookies()
            return True

        elif email is not None and password is not None:
            _log.info("Email and password are provided. Attemping login.")

            payload = {"mail_tel": email, "password": password}
            resp = self.session.http.post(_login_url, data=payload,
                                          params=_login_url_params)

            _log.debug("Login response code: {0}".format(resp.status_code))
            _log.trace(u"Login response body: {0}".format(resp.text))
            _log.debug("Cookies: {0}".format(
                self.session.http.cookies.get_dict()))

            if self.session.http.cookies.get("user_session") is None:
                try:
                    msg = extract_text(
                        resp.text, '<p class="notice__text">', "</p>")
                except Exception as e:
                    _log.debug(e)
                    msg = "unknown reason"
                _log.warn("Login failed. {0}".format(msg))
                return False
            else:
                _log.info("Logged in.")
                self.save_cookies()
                return True
        else:
            _log.warn(
                "Neither a email and password combination nor a user session "
                "token is provided. Cannot attempt login.")
            return False


class NicoHLSStream(HLSStream):

    def __init__(self, hls_stream, nicolive_plugin):
        super(NicoHLSStream, self).__init__(
            hls_stream.session,
            force_restart=hls_stream.force_restart,
            start_offset=hls_stream.start_offset,
            duration=hls_stream.duration,
            **hls_stream.args)
        # url is already in hls_stream.args

        self.nicolive_plugin = nicolive_plugin

    def open(self):
        reader = super(NicoHLSStream, self).open()
        self.nicolive_plugin.stream_reader = reader
        return reader


def extract_text(text, left, right):
    """Extract text from HTML"""
    result = re.findall("{0}(.*?){1}".format(left, right), text)
    if len(result) != 1:
        raise Exception("Failed to extract string. "
                        "Expected 1, found {0}".format(len(result)))
    return result[0]


def parse_proxy_url(purl):
    """Adapted from UStreamTV plugin (ustreamtv.py)"""

    proxy_options = {}
    if purl:
        p = urlparse(purl)
        proxy_options['proxy_type'] = p.scheme
        proxy_options['http_proxy_host'] = p.hostname
        if p.port:
            proxy_options['http_proxy_port'] = p.port
        if p.username:
            proxy_options['http_proxy_auth'] = \
                (unquote_plus(p.username), unquote_plus(p.password or ""))
    return proxy_options


__plugin__ = NicoLive



import time
from requests import Session, __build__ as requests_version
from requests.adapters import HTTPAdapter

from streamlink.packages.requests_file import FileAdapter
from streamlink.plugin.api import useragents

try:
    from requests.packages.urllib3.util import Timeout

    TIMEOUT_ADAPTER_NEEDED = requests_version < 0x020300
except ImportError:
    TIMEOUT_ADAPTER_NEEDED = False

try:
    from requests.packages import urllib3

    # We tell urllib3 to disable warnings about unverified HTTPS requests,
    # because in some plugins we have to do unverified requests intentionally.
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
except (ImportError, AttributeError):
    pass

from ...exceptions import PluginError
from ...utils import parse_json, parse_xml

__all__ = ["HTTPSession"]


def _parse_keyvalue_list(val):
    for keyvalue in val.split(";"):
        try:
            key, value = keyvalue.split("=", 1)
            yield key.strip(), value.strip()
        except ValueError:
            continue


class HTTPAdapterWithReadTimeout(HTTPAdapter):
    """This is a backport of the timeout behaviour from requests 2.3.0+
       where timeout is applied to both connect and read."""

    def get_connection(self, *args, **kwargs):
        conn = HTTPAdapter.get_connection(self, *args, **kwargs)

        # Override the urlopen method on this connection
        if not hasattr(conn.urlopen, "wrapped"):
            orig_urlopen = conn.urlopen

            def urlopen(*args, **kwargs):
                timeout = kwargs.pop("timeout", None)
                if isinstance(timeout, Timeout):
                    timeout = Timeout.from_float(timeout.connect_timeout)

                return orig_urlopen(*args, timeout=timeout, **kwargs)

            conn.urlopen = urlopen
            conn.urlopen.wrapped = True

        return conn


class HTTPSession(Session):
    def __init__(self, *args, **kwargs):
        Session.__init__(self, *args, **kwargs)

        if self.headers['User-Agent'].startswith('python-requests'):
            self.headers['User-Agent'] = useragents.FIREFOX

        self.timeout = 20.0

        if TIMEOUT_ADAPTER_NEEDED:
            self.mount("http://", HTTPAdapterWithReadTimeout())
            self.mount("https://", HTTPAdapterWithReadTimeout())

        self.mount('file://', FileAdapter())

    @classmethod
    def determine_json_encoding(cls, sample):
        """
        Determine which Unicode encoding the JSON text sample is encoded with

        RFC4627 (http://www.ietf.org/rfc/rfc4627.txt) suggests that the encoding of JSON text can be determined
        by checking the pattern of NULL bytes in first 4 octets of the text.
        :param sample: a sample of at least 4 bytes of the JSON text
        :return: the most likely encoding of the JSON text
        """
        nulls_at = [i for i, j in enumerate(bytearray(sample[:4])) if j == 0]
        if nulls_at == [0, 1, 2]:
            return "UTF-32BE"
        elif nulls_at == [0, 2]:
            return "UTF-16BE"
        elif nulls_at == [1, 2, 3]:
            return "UTF-32LE"
        elif nulls_at == [1, 3]:
            return "UTF-16LE"
        else:
            return "UTF-8"

    @classmethod
    def json(cls, res, *args, **kwargs):
        """Parses JSON from a response."""
        # if an encoding is already set then use the provided encoding
        if res.encoding is None:
            res.encoding = cls.determine_json_encoding(res.content[:4])
        return parse_json(res.text, *args, **kwargs)

    @classmethod
    def xml(cls, res, *args, **kwargs):
        """Parses XML from a response."""
        return parse_xml(res.text, *args, **kwargs)

    def parse_cookies(self, cookies, **kwargs):
        """Parses a semi-colon delimited list of cookies.

        Example: foo=bar;baz=qux
        """
        for name, value in _parse_keyvalue_list(cookies):
            self.cookies.set(name, value, **kwargs)

    def parse_headers(self, headers):
        """Parses a semi-colon delimited list of headers.

        Example: foo=bar;baz=qux
        """
        for name, value in _parse_keyvalue_list(headers):
            self.headers[name] = value

    def parse_query_params(self, cookies, **kwargs):
        """Parses a semi-colon delimited list of query parameters.

        Example: foo=bar;baz=qux
        """
        for name, value in _parse_keyvalue_list(cookies):
            self.params[name] = value

    def resolve_url(self, url):
        """Resolves any redirects and returns the final URL."""
        return self.get(url, stream=True).url

    def request(self, method, url, *args, **kwargs):
        acceptable_status = kwargs.pop("acceptable_status", [])
        exception = kwargs.pop("exception", PluginError)
        headers = kwargs.pop("headers", {})
        params = kwargs.pop("params", {})
        proxies = kwargs.pop("proxies", self.proxies)
        raise_for_status = kwargs.pop("raise_for_status", True)
        schema = kwargs.pop("schema", None)
        session = kwargs.pop("session", None)
        timeout = kwargs.pop("timeout", self.timeout)
        total_retries = kwargs.pop("retries", 0)
        retry_backoff = kwargs.pop("retry_backoff", 0.3)
        retry_max_backoff = kwargs.pop("retry_max_backoff", 10.0)
        retries = 0

        if session:
            headers.update(session.headers)
            params.update(session.params)

        while True:
            try:
                res = Session.request(self, method, url,
                                      headers=headers,
                                      params=params,
                                      timeout=timeout,
                                      proxies=proxies,
                                      *args, **kwargs)
                if raise_for_status and res.status_code not in acceptable_status:
                    res.raise_for_status()
                break
            except KeyboardInterrupt:
                raise
            except Exception as rerr:
                if retries >= total_retries:
                    err = exception("Unable to open URL: {url} ({err})".format(url=url,
                                                                               err=rerr))
                    err.err = rerr
                    raise err
                retries += 1
                # back off retrying, but only to a maximum sleep time
                delay = min(retry_max_backoff,
                            retry_backoff * (2 ** (retries - 1)))
                time.sleep(delay)

        if schema:
            res = schema.validate(res.text, name="response text", exception=PluginError)

        return res
